@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2006-2008, 2014, 2016-2017 by Hanna Knutsson            *
+ *   Copyright (C) 2006-2008, 2014, 2016-2020 by Hanna Knutsson            *
  *   hanna.knutsson@protonmail.com                                         *
  *                                                                         *
  *   This file is part of Eqonomize!.                                      *
@@ -24,6 +24,7 @@
 #include <QList>
 #include <QHash>
 #include <QString>
+#include <QStringList>
 #include <QCoreApplication>
 #include <QFile>
 #include <QVector>
@@ -39,7 +40,6 @@
 #define MONETARY_DECIMAL_PLACES 2
 #define SAVE_MONETARY_DECIMAL_PLACES 4
 #define QUANTITY_DECIMAL_PLACES 2
-#define CURRENCY_IS_PREFIX currency_symbol_precedes()
 #define IS_GREGORIAN_CALENDAR true
 
 class QProcess;
@@ -50,10 +50,7 @@ typedef enum {
 	TRANSACTION_CONVERSION_LATEST_RATE
 } TransactionConversionRateDate;
 
-int currency_frac_digits();
-bool currency_symbol_precedes();
 bool is_zero(double);
-QString format_money(double v, int precision);
 
 void read_id(QXmlStreamAttributes *attr, qlonglong &id, int &rev1, int &rev2);
 void write_id(QXmlStreamAttributes *attr, qlonglong &id, int &rev1, int &rev2);
@@ -69,30 +66,30 @@ template<class type> class TransactionList : public EqonomizeList<type> {
 	public:
 		TransactionList() : EqonomizeList<type>() {};
 		void sort() {
-			qSort(QList<type>::begin(), QList<type>::end(), transaction_list_less_than);
+			std::sort(QList<type>::begin(), QList<type>::end(), transaction_list_less_than);
 		}
 		void inSort(type value) {
-			QList<type>::insert(qLowerBound(QList<type>::begin(), QList<type>::end(), value, transaction_list_less_than), value);
+			QList<type>::insert(std::lower_bound(QList<type>::begin(), QList<type>::end(), value, transaction_list_less_than), value);
 		}
 };
 template<class type> class SplitTransactionList : public EqonomizeList<type> {
 	public:
 		SplitTransactionList() : EqonomizeList<type>() {};
 		void sort() {
-			qSort(QList<type>::begin(), QList<type>::end(), split_list_less_than);
+			std::sort(QList<type>::begin(), QList<type>::end(), split_list_less_than);
 		}
 		void inSort(type value) {
-			QList<type>::insert(qLowerBound(QList<type>::begin(), QList<type>::end(), value, split_list_less_than), value);
+			QList<type>::insert(std::lower_bound(QList<type>::begin(), QList<type>::end(), value, split_list_less_than), value);
 		}
 };
 template<class type> class ScheduledTransactionList : public EqonomizeList<type> {
 	public:
 		ScheduledTransactionList() : EqonomizeList<type>() {};
 		void sort() {
-			qSort(QList<type>::begin(), QList<type>::end(), schedule_list_less_than);
+			std::sort(QList<type>::begin(), QList<type>::end(), schedule_list_less_than);
 		}
 		void inSort(type value) {
-			QList<type>::insert(qLowerBound(QList<type>::begin(), QList<type>::end(), value, schedule_list_less_than), value);
+			QList<type>::insert(std::lower_bound(QList<type>::begin(), QList<type>::end(), value, schedule_list_less_than), value);
 		}
 
 };
@@ -100,20 +97,20 @@ template<class type> class SecurityList : public EqonomizeList<type> {
 	public:
 		SecurityList() : EqonomizeList<type>() {};
 		void sort() {
-			qSort(QList<type>::begin(), QList<type>::end(), security_list_less_than);
+			std::sort(QList<type>::begin(), QList<type>::end(), security_list_less_than);
 		}
 		void inSort(type value) {
-			QList<type>::insert(qLowerBound(QList<type>::begin(), QList<type>::end(), value, security_list_less_than), value);
+			QList<type>::insert(std::lower_bound(QList<type>::begin(), QList<type>::end(), value, security_list_less_than), value);
 		}
 };
 template<class type> class SecurityTradeList : public EqonomizeList<type> {
 	public:
 		SecurityTradeList() : EqonomizeList<type>() {};
 		void sort() {
-			qSort(QList<type>::begin(), QList<type>::end(), trade_list_less_than);
+			std::sort(QList<type>::begin(), QList<type>::end(), trade_list_less_than);
 		}
 		void inSort(type value) {
-			QList<type>::insert(qLowerBound(QList<type>::begin(), QList<type>::end(), value, trade_list_less_than), value);
+			QList<type>::insert(std::lower_bound(QList<type>::begin(), QList<type>::end(), value, trade_list_less_than), value);
 		}
 };
 
@@ -123,9 +120,9 @@ struct BudgetSynchronization {
 	int revision;
 	void clear() {
 		autosync = false;
-		url = QString::null;
-		download = QString::null;
-		upload = QString::null;
+		url = QString();
+		download = QString();
+		upload = QString();
 		revision = 0;
 	}
 	bool isComplete() {
@@ -136,29 +133,36 @@ struct BudgetSynchronization {
 class Budget {
 
 	Q_DECLARE_TR_FUNCTIONS(Budget)
-	
+
 	protected:
-	
-		int i_quotation_decimals, i_share_decimals, i_budget_day, i_opened_revision, i_revision;
-		bool b_record_new_accounts, b_record_new_securities, b_default_currency_changed, b_currency_modified;
+
+		int i_quotation_decimals, i_share_decimals, i_budget_day, i_budget_month, i_opened_revision, i_revision;
+		bool b_record_new_tags, b_record_new_accounts, b_record_new_securities, b_default_currency_changed, b_currency_modified;
 		TransactionConversionRateDate i_tcrd;
-		
+
 		qlonglong last_id;
-		
+
 		Currency *default_currency;
-		
+
 		QNetworkReply *syncReply;
 		QProcess *syncProcess;
 
 	public:
-	
+
 		BudgetSynchronization *o_sync;
 		Currency *currency_euro;
 		QNetworkAccessManager nam;
 
+		QString monetary_decimal_separator, monetary_group_separator, monetary_positive_sign, monetary_negative_sign;
+		QByteArray monetary_group_format;
+		bool currency_symbol_precedes, currency_symbol_precedes_neg, currency_code_precedes, currency_code_precedes_neg, currency_symbol_space, currency_code_space, currency_symbol_space_neg, currency_code_space_neg;
+		int monetary_sign_p_symbol_neg, monetary_sign_p_symbol_pos, monetary_sign_p_code_neg, monetary_sign_p_code_pos, monetary_decimal_places;
+		QString decimal_separator, group_separator, positive_sign, negative_sign;
+		QByteArray group_format;
+
 		Budget();
 		~Budget();
-		
+
 		void loadCurrencies();
 		void loadGlobalCurrencies();
 		void loadLocalCurrencies();
@@ -167,7 +171,7 @@ class Budget {
 		QString loadMyCurrencyNetData(QByteArray data);
 		QString loadMyCurrencyNetHtml(QByteArray data);
 		QString saveCurrencies();
-		
+
 		TransactionConversionRateDate defaultTransactionConversionRateDate() const;
 		void setDefaultTransactionConversionRateDate(TransactionConversionRateDate tcrd);
 
@@ -180,14 +184,16 @@ class Budget {
 		bool sync(QString &error, QString &errors, bool do_upload = true, bool on_load = false);
 		QString syncUpload(QString filename);
 		bool autosyncEnabled() const;
-		
+
 		void clear();
-		
+
 		QString formatMoney(double v, int precision = -1, bool show_currency = true);
+		QString formatValue(double v, int precision = 2, bool always_show_sign = false);
+		QString formatValue(int v, int precision = 0, bool always_show_sign = false);
 
 		void addTransaction(Transaction*);
 		void removeTransactions(Transactions*, bool keep = false);
-		
+
 		void addTransactions(Transactions*);
 		void removeTransaction(Transaction*, bool keep = false);
 
@@ -196,20 +202,20 @@ class Budget {
 
 		void addSplitTransaction(SplitTransaction*);
 		void removeSplitTransaction(SplitTransaction*, bool keep = false);
-		
+
 		void addAccount(Account*);
 		void removeAccount(Account*, bool keep = false);
 		void accountModified(Account*);
 
 		bool accountHasTransactions(Account*, bool check_subs = true);
 		void moveTransactions(Account*, Account*, bool move_from_subs = true);
-		
-		Transaction *findDuplicateTransaction(Transaction *trans); 
+
+		Transaction *findDuplicateTransaction(Transaction *trans);
 
 		void addSecurity(Security*);
 		void removeSecurity(Security*, bool keep = false);
 		Security *findSecurity(QString name);
-		
+
 		int defaultQuotationDecimals() const;
 		int defaultShareDecimals() const;
 		void setDefaultQuotationDecimals(int new_decimals);
@@ -237,7 +243,7 @@ class Budget {
 		ExpensesAccount *findExpensesAccount(QString name);
 		IncomesAccount *findIncomesAccount(QString name, CategoryAccount *parent_acc);
 		ExpensesAccount *findExpensesAccount(QString name, CategoryAccount *parent_acc);
-		
+
 		Currency *defaultCurrency();
 		void setDefaultCurrency(Currency*);
 		bool resetDefaultCurrency();
@@ -251,10 +257,10 @@ class Budget {
 		bool currenciesModified();
 		void resetCurrenciesModified();
 		void currencyModified(Currency*);
-		
+
 		qlonglong getNewId();
 		int revision();
-		
+
 		AccountList<IncomesAccount*> incomesAccounts;
 		AccountList<ExpensesAccount*> expensesAccounts;
 		AccountList<AssetsAccount*> assetsAccounts;
@@ -270,12 +276,12 @@ class Budget {
 		SecurityList<Security*> securities;
 		SecurityTradeList<SecurityTrade*> securityTrades;
 		CurrencyList<Currency*> currencies;
-		
+
 		IncomesAccount *null_incomes_account;
-		
+
 		void setRecordNewAccounts(bool rna);
 		QVector<Account*> newAccounts;
-		
+
 		void setRecordNewSecurities(bool rns);
 		QVector<Security*> newSecurities;
 
@@ -283,10 +289,14 @@ class Budget {
 		QHash<qlonglong, ExpensesAccount*> expensesAccounts_id;
 		QHash<qlonglong, AssetsAccount*> assetsAccounts_id;
 		QHash<qlonglong, Security*> securities_id;
-		
+
+		Transactions *getTransaction(qlonglong tid);
+
 		void setBudgetDay(int day_of_month);
 		int budgetDay() const;
-		
+		void setBudgetMonth(int month_of_year);
+		int budgetMonth() const;
+
 		bool isSameBudgetMonth(const QDate &date1, const QDate &date2) const;
 		int daysInBudgetMonth(const QDate &date) const;
 		int daysInBudgetYear(const QDate &date) const;
@@ -294,6 +304,8 @@ class Budget {
 		int dayOfBudgetMonth(const QDate &date) const;
 		int budgetMonth(const QDate &date) const;
 		int budgetYear(const QDate &date) const;
+		QString budgetYearString(const QDate &date, bool short_format = false) const;
+		QString budgetYearString(int year, bool short_format = false) const;
 		bool isFirstBudgetDay(const QDate &date) const;
 		bool isLastBudgetDay(const QDate &date) const;
 		void addBudgetMonthsSetLast(QDate &date, int months) const;
@@ -311,6 +323,23 @@ class Budget {
 		double monthsBetweenDates(const QDate &date1, const QDate &date2, bool use_budget_months = true);
 		int calendarMonthsBetweenDates(const QDate &date1, const QDate &date2, bool use_budget_months = true);
 
+		int getAccountType(const QString &type, bool localized = false, bool plural = false);
+		QString getAccountTypeName(int at_type, bool localized = false, bool plural = false);
+		bool accountTypeIsDebt(int at_type);
+		bool accountTypeIsCreditCard(int at_type);
+		bool accountTypeIsSecurities(int at_type);
+		bool accountTypeIsLiabilities(int at_type);
+		bool accountTypeIsOther(int at_type);
+
+		void tagAdded(const QString &tag);
+		void tagRemoved(const QString &tag);
+		QString findTag(const QString &tag);
+
+		void setRecordNewTags(bool rnt);
+		QVector<QString> newTags;
+
+		QStringList tags;
+
 };
 
 #endif
