@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2006-2008, 2014, 2016 by Hanna Knutsson                 *
+ *   Copyright (C) 2006-2008, 2014, 2016-2020 by Hanna Knutsson            *
  *   hanna.knutsson@protonmail.com                                         *
  *                                                                         *
  *   This file is part of Eqonomize!.                                      *
@@ -29,7 +29,8 @@
 #include "transaction.h"
 #include "recurrence.h"
 #include "eqonomize.h"
-
+#include "eqonomizevalueedit.h"
+#include "overtimereport.h"
 
 #ifdef QT_CHARTS_LIB
 #include <QtCharts/QLegendMarker>
@@ -65,6 +66,7 @@
 #include <QPrintDialog>
 #include <QPixmap>
 #include <QRadioButton>
+#include <QButtonGroup>
 #include <QResizeEvent>
 #include <QString>
 #include <QVBoxLayout>
@@ -114,6 +116,14 @@ CategoriesComparisonChart::CategoriesComparisonChart(Budget *budg, QWidget *pare
 	themeCombo->addItem("Qt", QChart::ChartThemeQt);
 	buttons->addWidget(themeCombo);
 #endif
+	QButtonGroup *group = new QButtonGroup(this);
+	percentButton = new QRadioButton("%", this);
+	percentButton->setChecked(true);
+	group->addButton(percentButton, 1);
+	buttons->addWidget(percentButton);
+	valueButton = new QRadioButton(budget->defaultCurrency()->symbol(true), this);
+	group->addButton(valueButton, 2);
+	buttons->addWidget(valueButton);
 	buttons->addStretch();
 	saveButton = new QPushButton(tr("Save As…"), this);
 	saveButton->setAutoDefault(false);
@@ -126,6 +136,8 @@ CategoriesComparisonChart::CategoriesComparisonChart(Budget *budg, QWidget *pare
 	chart = new QChart();
 	view = new QChartView(chart, this);
 	series = NULL;
+	axis_x = NULL;
+	axis_y = NULL;
 #else
 	scene = NULL;
 	view = new QGraphicsView(this);
@@ -133,20 +145,20 @@ CategoriesComparisonChart::CategoriesComparisonChart(Budget *budg, QWidget *pare
 	view->setRenderHint(QPainter::Antialiasing, true);
 	layout->addWidget(view);
 	QWidget *settingsWidget = new QWidget(this);
-	QHBoxLayout *settingsLayout = new QHBoxLayout(settingsWidget);	
+	QHBoxLayout *settingsLayout = new QHBoxLayout(settingsWidget);
 
 	QHBoxLayout *choicesLayout = new QHBoxLayout();
 	settingsLayout->addLayout(choicesLayout);
 	fromButton = new QCheckBox(tr("From"), settingsWidget);
 	fromButton->setChecked(true);
 	choicesLayout->addWidget(fromButton);
-	fromEdit = new QDateEdit(settingsWidget);
+	fromEdit = new EqonomizeDateEdit(settingsWidget);
 	fromEdit->setCalendarPopup(true);
 	choicesLayout->addWidget(fromEdit);
 	choicesLayout->setStretchFactor(fromEdit, 1);
 	choicesLayout->addWidget(new QLabel(tr("To"), settingsWidget));
-	toEdit = new QDateEdit(settingsWidget);
-	toEdit->setCalendarPopup(true);	
+	toEdit = new EqonomizeDateEdit(settingsWidget);
+	toEdit->setCalendarPopup(true);
 	choicesLayout->addWidget(toEdit);
 	choicesLayout->setStretchFactor(toEdit, 1);
 	prevYearButton = new QPushButton(LOAD_ICON("eqz-previous-year"), "", settingsWidget);
@@ -158,7 +170,7 @@ CategoriesComparisonChart::CategoriesComparisonChart(Budget *budg, QWidget *pare
 	nextYearButton = new QPushButton(LOAD_ICON("eqz-next-year"), "", settingsWidget);
 	choicesLayout->addWidget(nextYearButton);
 	settingsLayout->addSpacing(12);
-	settingsLayout->setStretchFactor(choicesLayout, 2);
+	settingsLayout->setStretchFactor(choicesLayout, 1);
 
 	QHBoxLayout *typeLayout = new QHBoxLayout();
 	settingsLayout->addLayout(typeLayout);
@@ -179,24 +191,17 @@ CategoriesComparisonChart::CategoriesComparisonChart(Budget *budg, QWidget *pare
 		sourceCombo->addItem(tr("Incomes: %1").arg(account->nameWithParent()));
 	}
 	typeLayout->addWidget(sourceCombo);
-	typeLayout->setStretchFactor(sourceCombo, 2);
-	accountCombo = new QComboBox(settingsWidget);
-	accountCombo->setEditable(false);
-	accountCombo->addItem(tr("All Accounts"), qVariantFromValue(NULL));
-	for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
-		AssetsAccount *account = *it;
-		if(account != budget->balancingAccount && account->accountType() != ASSETS_TYPE_SECURITIES) {
-			accountCombo->addItem(account->name(), qVariantFromValue((void*) account));
-		}
-	}
+	typeLayout->setStretchFactor(sourceCombo, 1);
+	accountCombo = new AccountsCombo(budget, settingsWidget, true);
+	accountCombo->updateAccounts(ACCOUNT_TYPE_ASSETS);
 	typeLayout->addWidget(accountCombo);
 	typeLayout->setStretchFactor(accountCombo, 1);
-	settingsLayout->setStretchFactor(typeLayout, 1);
+	settingsLayout->setStretchFactor(typeLayout, 2);
 
 	current_account = NULL;
 
 	layout->addWidget(settingsWidget);
-	
+
 	resetOptions();
 
 #ifdef QT_CHARTS_LIB
@@ -204,7 +209,7 @@ CategoriesComparisonChart::CategoriesComparisonChart(Budget *budg, QWidget *pare
 	connect(typeCombo, SIGNAL(activated(int)), this, SLOT(typeChanged(int)));
 #endif
 	connect(sourceCombo, SIGNAL(activated(int)), this, SLOT(sourceChanged(int)));
-	connect(accountCombo, SIGNAL(activated(int)), this, SLOT(updateDisplay()));
+	connect(accountCombo, SIGNAL(selectedAccountsChanged()), this, SLOT(updateDisplay()));
 	connect(prevMonthButton, SIGNAL(clicked()), this, SLOT(prevMonth()));
 	connect(nextMonthButton, SIGNAL(clicked()), this, SLOT(nextMonth()));
 	connect(prevYearButton, SIGNAL(clicked()), this, SLOT(prevYear()));
@@ -215,6 +220,7 @@ CategoriesComparisonChart::CategoriesComparisonChart(Budget *budg, QWidget *pare
 	connect(toEdit, SIGNAL(dateChanged(const QDate&)), this, SLOT(toChanged(const QDate&)));
 	connect(saveButton, SIGNAL(clicked()), this, SLOT(save()));
 	connect(printButton, SIGNAL(clicked()), this, SLOT(print()));
+	connect(group, SIGNAL(buttonToggled(int, bool)), this, SLOT(valueTypeToggled(int, bool)));
 
 }
 
@@ -222,17 +228,24 @@ CategoriesComparisonChart::~CategoriesComparisonChart() {
 }
 
 void CategoriesComparisonChart::resetOptions() {
-#ifdef QT_CHARTS_LIB
 	QSettings settings;
 	settings.beginGroup("CategoriesComparisonChart");
+#ifdef QT_CHARTS_LIB
 	int theme = settings.value("theme", -1).toInt();
-	int index = themeCombo->findData(qVariantFromValue(theme));
+	int index = themeCombo->findData(QVariant::fromValue(theme));
 	if(index < 0) index = 0;
 	themeCombo->setCurrentIndex(index);
 	typeCombo->setCurrentIndex(0);
 	chart->setTheme(theme >= 0 ? (QChart::ChartTheme) theme : QChart::ChartThemeBlueNcs);
-	settings.endGroup();
 #endif
+	int value_type = settings.value("valueType", 2).toInt();
+	settings.endGroup();
+	valueButton->blockSignals(true);
+	percentButton->blockSignals(true);
+	if(value_type == 2) valueButton->setChecked(true);
+	else percentButton->setChecked(true);
+	valueButton->blockSignals(false);
+	percentButton->blockSignals(false);
 	QDate first_date;
 	for(TransactionList<Transaction*>::const_iterator it = budget->transactions.constBegin(); it != budget->transactions.constEnd(); ++it) {
 		Transaction *trans = *it;
@@ -254,7 +267,9 @@ void CategoriesComparisonChart::resetOptions() {
 	toEdit->setDate(to_date);
 	fromEdit->blockSignals(false);
 	toEdit->blockSignals(false);
-	accountCombo->setCurrentIndex(0);
+	accountCombo->blockSignals(true);
+	accountCombo->deselectAll();
+	accountCombo->blockSignals(false);
 	sourceCombo->setCurrentIndex(0);
 	sourceChanged(0);
 }
@@ -278,8 +293,13 @@ void CategoriesComparisonChart::toChanged(const QDate &date) {
 		error = true;
 	}
 	if(!error && fromEdit->date() > date) {
-		if(fromButton->isChecked() && fromButton->isEnabled()) {
+		/*if(fromButton->isChecked() && fromButton->isEnabled()) {
 			QMessageBox::critical(this, tr("Error"), tr("To date is before from date."));
+		}*/
+		if(budget->isFirstBudgetDay(to_date)) {
+			from_date = budget->firstBudgetDay(date);
+		} else {
+			from_date = date.addDays(-from_date.daysTo(to_date));
 		}
 		from_date = date;
 		fromEdit->blockSignals(true);
@@ -304,8 +324,13 @@ void CategoriesComparisonChart::fromChanged(const QDate &date) {
 		error = true;
 	}
 	if(!error && date > toEdit->date()) {
-		QMessageBox::critical(this, tr("Error"), tr("From date is after to date."));
-		to_date = date;
+		//QMessageBox::critical(this, tr("Error"), tr("From date is after to date."));
+		if(budget->isLastBudgetDay(to_date)) {
+			to_date = budget->lastBudgetDay(date);
+		} else {
+			to_date = date.addDays(from_date.daysTo(to_date));
+		}
+		if(to_date > QDate::currentDate() && from_date <= QDate::currentDate()) to_date = QDate::currentDate();
 		toEdit->blockSignals(true);
 		toEdit->setDate(to_date);
 		toEdit->blockSignals(false);
@@ -381,7 +406,7 @@ void CategoriesComparisonChart::onFilterSelected(QString filter) {
 void CategoriesComparisonChart::save() {
 #ifdef QT_CHARTS_LIB
 	QGraphicsScene *scene = chart->scene();
-#endif	
+#endif
 	if(!scene) return;
 	QMimeDatabase db;
 	QMimeType image_mime = db.mimeTypeForName("image/*");
@@ -580,15 +605,17 @@ void CategoriesComparisonChart::updateDisplay() {
 	QMap<QString, QString> desc_map;
 	QMap<QString, double> desc_values;
 	QMap<QString, double> desc_counts;
-	QStringList desc_order; 
+	QStringList desc_order;
 
 	current_account = NULL;
-	AssetsAccount *current_assets = selectedAccount();
-	
+
+	bool assets_selected = accountCombo->isEnabled() && !accountCombo->allAccountsSelected();
+	bool single_assets = assets_selected && accountCombo->selectedAccounts().count() == 1;
+
 	bool include_subs = false;
-	
+
 	QString title_string = tr("Expenses");
-	
+
 	QDate first_date;
 	bool first_date_reached = false;
 	if(sourceCombo->currentIndex() == 4) {
@@ -614,7 +641,7 @@ void CategoriesComparisonChart::updateDisplay() {
 		}
 		case 0: {
 			type = ACCOUNT_TYPE_EXPENSES;
-			if(current_assets) title_string = tr("Expenses, %1").arg(current_assets->name());
+			if(assets_selected) title_string = tr("Expenses, %1").arg(accountCombo->selectedAccountsText(2));
 			for(AccountList<ExpensesAccount*>::const_iterator it = budget->expensesAccounts.constBegin(); it != budget->expensesAccounts.constEnd(); ++it) {
 				CategoryAccount *account = *it;
 				if(include_subs || !account->parentCategory()) {
@@ -628,7 +655,7 @@ void CategoriesComparisonChart::updateDisplay() {
 			include_subs = true;
 		}
 		case 2: {
-			if(current_assets) title_string = tr("Incomes, %1").arg(current_assets->name());
+			if(assets_selected) title_string = tr("Incomes, %1").arg(accountCombo->selectedAccountsText(2));
 			else title_string = tr("Incomes");
 			type = ACCOUNT_TYPE_INCOMES;
 			for(AccountList<IncomesAccount*>::const_iterator it = budget->incomesAccounts.constBegin(); it != budget->incomesAccounts.constEnd(); ++it) {
@@ -642,7 +669,6 @@ void CategoriesComparisonChart::updateDisplay() {
 		}
 		case 4: {
 			title_string = tr("Accounts");
-			current_assets = NULL;
 			type = ACCOUNT_TYPE_ASSETS;
 			for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
 				AssetsAccount *account = *it;
@@ -664,9 +690,9 @@ void CategoriesComparisonChart::updateDisplay() {
 			else current_account = budget->incomesAccounts.at(i - budget->expensesAccounts.count());
 			if(current_account) {
 				type = current_account->type();
-				if(current_assets) {
-					if(type == ACCOUNT_TYPE_EXPENSES) title_string = tr("Expenses, %2: %1").arg(current_account->nameWithParent()).arg(current_assets->name());
-					else title_string = tr("Incomes, %2: %1").arg(current_account->nameWithParent()).arg(current_assets->name());
+				if(assets_selected) {
+					if(type == ACCOUNT_TYPE_EXPENSES) title_string = tr("Expenses, %2: %1").arg(current_account->nameWithParent()).arg(accountCombo->selectedAccountsText(2));
+					else title_string = tr("Incomes, %2: %1").arg(current_account->nameWithParent()).arg(accountCombo->selectedAccountsText(2));
 				} else {
 					if(type == ACCOUNT_TYPE_EXPENSES) title_string = tr("Expenses: %1").arg(current_account->nameWithParent());
 					else title_string = tr("Incomes: %1").arg(current_account->nameWithParent());
@@ -684,7 +710,7 @@ void CategoriesComparisonChart::updateDisplay() {
 					for(TransactionList<Transaction*>::const_iterator it = budget->transactions.constEnd(); it != budget->transactions.constBegin();) {
 						--it;
 						Transaction *trans = *it;
-						if(trans->date() <= to_date && (!current_assets || trans->relatesToAccount(current_assets))) {
+						if(trans->date() <= to_date && (!assets_selected || accountCombo->testTransactionRelation(trans))) {
 							if(trans->date() < first_date) break;
 							if((trans->fromAccount() == current_account || trans->toAccount() == current_account) && !desc_map.contains(trans->description().toLower())) {
 								desc_map[trans->description().toLower()] = trans->description();
@@ -697,29 +723,29 @@ void CategoriesComparisonChart::updateDisplay() {
 			}
 		}
 	}
-	
+
 	Currency *currency = budget->defaultCurrency();
-	if(current_assets) currency = current_assets->currency();
+	if(single_assets) currency = ((AssetsAccount*) accountCombo->selectedAccounts()[0])->currency();
 
 	for(TransactionList<Transaction*>::const_iterator it = budget->transactions.constBegin(); it != budget->transactions.constEnd(); ++it) {
 		Transaction *trans = *it;
 		if(!first_date_reached && trans->date() >= first_date) first_date_reached = true;
 		else if(first_date_reached && trans->date() > to_date) break;
-		if(first_date_reached && (!current_assets || trans->relatesToAccount(current_assets))) {
+		if(first_date_reached && (!assets_selected || accountCombo->testTransactionRelation(trans))) {
 			if(current_account && !include_subs) {
 				if(trans->fromAccount() == current_account) {
-					if(type == ACCOUNT_TYPE_EXPENSES) {desc_values[trans->description().toLower()] -= trans->value(!current_assets);}
-					else {desc_values[trans->description().toLower()] += trans->value(!current_assets);}
+					if(type == ACCOUNT_TYPE_EXPENSES) {desc_values[trans->description().toLower()] -= trans->value(!single_assets);}
+					else {desc_values[trans->description().toLower()] += trans->value(!single_assets);}
 					desc_counts[trans->description().toLower()] += trans->quantity();
 				} else if(trans->toAccount() == current_account) {
-					if(type == ACCOUNT_TYPE_EXPENSES) {desc_values[trans->description().toLower()] += trans->value(!current_assets);}
-					else {desc_values[trans->description().toLower()] -= trans->value(!current_assets);}
+					if(type == ACCOUNT_TYPE_EXPENSES) {desc_values[trans->description().toLower()] += trans->value(!single_assets);}
+					else {desc_values[trans->description().toLower()] -= trans->value(!single_assets);}
 					desc_counts[trans->description().toLower()] += trans->quantity();
 				}
 			} else if(type == ACCOUNT_TYPE_ASSETS) {
 				if(trans->fromAccount()->type() == ACCOUNT_TYPE_ASSETS && trans->fromAccount() != budget->balancingAccount) {
 					if(((AssetsAccount*) trans->fromAccount())->accountType() != ASSETS_TYPE_SECURITIES) {
-						values[trans->fromAccount()] -= trans->fromValue(!current_assets);
+						values[trans->fromAccount()] -= trans->fromValue(!single_assets);
 					}
 					if(trans->toAccount() != budget->balancingAccount) {
 						counts[trans->fromAccount()] += trans->quantity();
@@ -727,7 +753,7 @@ void CategoriesComparisonChart::updateDisplay() {
 				}
 				if(trans->toAccount()->type() == ACCOUNT_TYPE_ASSETS && trans->toAccount() != budget->balancingAccount) {
 					if(((AssetsAccount*) trans->toAccount())->accountType() != ASSETS_TYPE_SECURITIES) {
-						values[trans->toAccount()] += trans->toValue(!current_assets);
+						values[trans->toAccount()] += trans->toValue(!single_assets);
 					}
 					if(trans->fromAccount() != budget->balancingAccount) {
 						counts[trans->toAccount()] += trans->quantity();
@@ -740,16 +766,16 @@ void CategoriesComparisonChart::updateDisplay() {
 				if(!include_subs) to_account = to_account->topAccount();
 				if((!current_account || to_account->topAccount() == current_account || from_account->topAccount() == current_account) && (to_account->type() == type || from_account->type() == type)) {
 					if(from_account->type() == ACCOUNT_TYPE_EXPENSES) {
-						values[from_account] -= trans->value(!current_assets);
+						values[from_account] -= trans->value(!single_assets);
 						counts[from_account] += trans->quantity();
 					} else if(from_account->type() == ACCOUNT_TYPE_INCOMES) {
-						values[from_account] += trans->value(!current_assets);
+						values[from_account] += trans->value(!single_assets);
 						counts[from_account] += trans->quantity();
 					} else if(to_account->type() == ACCOUNT_TYPE_EXPENSES) {
-						values[to_account] += trans->value(!current_assets);
+						values[to_account] += trans->value(!single_assets);
 						counts[to_account] += trans->quantity();
 					} else if(to_account->type() == ACCOUNT_TYPE_INCOMES) {
-						values[to_account] -= trans->value(!current_assets);
+						values[to_account] -= trans->value(!single_assets);
 						counts[to_account] += trans->quantity();
 					}
 				}
@@ -773,18 +799,18 @@ void CategoriesComparisonChart::updateDisplay() {
 			trans = (Transaction*) strans->transaction();
 		}
 		if(!first_date_reached && trans->date() >= first_date) first_date_reached = true;
-		else if(first_date_reached && trans->date() > to_date) break;
-		if(first_date_reached && (!current_assets || trans->relatesToAccount(current_assets))) {
+		if(first_date_reached && trans->date() > to_date) break;
+		if(first_date_reached && (!assets_selected || accountCombo->testTransactionRelation(trans))) {
 			if(current_account && !include_subs) {
 				if(trans->fromAccount() == current_account) {
 					int count = strans->recurrence() ? strans->recurrence()->countOccurrences(first_date, to_date) : 1;
-					if(type == ACCOUNT_TYPE_EXPENSES) {desc_values[trans->description().toLower()] -= trans->value(!current_assets) * count;}
-					else {desc_values[trans->description().toLower()] += trans->value(!current_assets) * count;}
+					if(type == ACCOUNT_TYPE_EXPENSES) {desc_values[trans->description().toLower()] -= trans->value(!single_assets) * count;}
+					else {desc_values[trans->description().toLower()] += trans->value(!single_assets) * count;}
 					desc_counts[trans->description().toLower()] += count *  trans->quantity();
 				} else if(trans->toAccount() == current_account) {
 					int count = strans->recurrence() ? strans->recurrence()->countOccurrences(first_date, to_date) : 1;
-					if(type == ACCOUNT_TYPE_EXPENSES) {desc_values[trans->description().toLower()] += trans->value(!current_assets) * count;}
-					else {desc_values[trans->description().toLower()] -= trans->value(!current_assets) * count;}
+					if(type == ACCOUNT_TYPE_EXPENSES) {desc_values[trans->description().toLower()] += trans->value(!single_assets) * count;}
+					else {desc_values[trans->description().toLower()] -= trans->value(!single_assets) * count;}
 					desc_counts[trans->description().toLower()] += count *  trans->quantity();
 				}
 			} else if(type == ACCOUNT_TYPE_ASSETS) {
@@ -794,7 +820,7 @@ void CategoriesComparisonChart::updateDisplay() {
 						counts[trans->fromAccount()] += count *  trans->quantity();
 					}
 					if(((AssetsAccount*) trans->fromAccount())->accountType() != ASSETS_TYPE_SECURITIES) {
-						values[trans->fromAccount()] -= trans->fromValue(!current_assets) * count;
+						values[trans->fromAccount()] -= trans->fromValue(!single_assets) * count;
 					}
 				}
 				if(trans->toAccount()->type() == ACCOUNT_TYPE_ASSETS && trans->toAccount() != budget->balancingAccount) {
@@ -803,7 +829,7 @@ void CategoriesComparisonChart::updateDisplay() {
 						counts[trans->toAccount()] += count *  trans->quantity();
 					}
 					if(((AssetsAccount*) trans->toAccount())->accountType() != ASSETS_TYPE_SECURITIES) {
-						values[trans->toAccount()] += trans->toValue(!current_assets) * count;
+						values[trans->toAccount()] += trans->toValue(!single_assets) * count;
 					}
 				}
 			} else {
@@ -815,19 +841,19 @@ void CategoriesComparisonChart::updateDisplay() {
 					if(from_account->type() == ACCOUNT_TYPE_EXPENSES) {
 						int count = strans->recurrence() ? strans->recurrence()->countOccurrences(first_date, to_date) : 1;
 						counts[trans->fromAccount()] += count *  trans->quantity();
-						values[trans->fromAccount()] -= trans->value(!current_assets) * count;
+						values[trans->fromAccount()] -= trans->value(!single_assets) * count;
 					} else if(from_account->type() == ACCOUNT_TYPE_INCOMES) {
 						int count = strans->recurrence() ? strans->recurrence()->countOccurrences(first_date, to_date) : 1;
 						counts[trans->fromAccount()] += count;
-						values[trans->fromAccount()] += trans->value(!current_assets) * count;
+						values[trans->fromAccount()] += trans->value(!single_assets) * count;
 					} else if(to_account->type() == ACCOUNT_TYPE_EXPENSES) {
 						int count = strans->recurrence() ? strans->recurrence()->countOccurrences(first_date, to_date) : 1;
 						counts[trans->toAccount()] += count *  trans->quantity();
-						values[trans->toAccount()] += trans->value(!current_assets) * count;
+						values[trans->toAccount()] += trans->value(!single_assets) * count;
 					} else if(to_account->type() == ACCOUNT_TYPE_INCOMES) {
 						int count = strans->recurrence() ? strans->recurrence()->countOccurrences(first_date, to_date) : 1;
 						counts[trans->toAccount()] += count;
-						values[trans->toAccount()] -= trans->value(!current_assets) * count;
+						values[trans->toAccount()] -= trans->value(!single_assets) * count;
 					}
 				}
 			}
@@ -841,7 +867,7 @@ void CategoriesComparisonChart::updateDisplay() {
 	if(type == ACCOUNT_TYPE_ASSETS) {
 		for(SecurityList<Security*>::const_iterator it = budget->securities.constBegin(); it != budget->securities.constEnd(); ++it) {
 			Security *security = *it;
-			double val = security->value(to_date, true);
+			double val = security->value(to_date, -1);
 			values[security->account()] += val;
 		}
 	}
@@ -854,11 +880,11 @@ void CategoriesComparisonChart::updateDisplay() {
 #else
 	int chart_type = 1;
 #endif
-	
+
 	Account *account = NULL;
 	QMap<QString, double>::iterator it_desc = desc_values.begin();
 	QMap<QString, double>::iterator it_desc_end = desc_values.end();
-	
+
 	double value = 0.0;
 	double vmax = 0.0;
 	double remaining_value = 0.0;
@@ -871,7 +897,7 @@ void CategoriesComparisonChart::updateDisplay() {
 				for(int i = 0; i < desc_order.count(); i++) {
 					if(it_desc.value() > desc_values[desc_order.at(i)]) {
 						desc_order.insert(i, it_desc.key());
-						
+
 						b = true;
 						break;
 					}
@@ -881,7 +907,7 @@ void CategoriesComparisonChart::updateDisplay() {
 			++it_desc;
 		}
 	}
-	
+
 	if((chart_type == 1 && desc_order.size() > 9) || (chart_type == 2 && desc_order.size() > 12)) {
 		while((chart_type == 1 && desc_order.size() > 8) || (chart_type == 2 && desc_order.size() > 11)) {
 			remaining_value += desc_values[desc_order.back()];
@@ -960,7 +986,7 @@ void CategoriesComparisonChart::updateDisplay() {
 			for(int i = 0; i < account_order.count(); i++) {
 				if(values[account] > values[account_order.at(i)]) {
 					account_order.insert(i, account);
-					
+
 					b = true;
 					break;
 				}
@@ -1033,7 +1059,7 @@ void CategoriesComparisonChart::updateDisplay() {
 	QPieSeries *pie_series = NULL;
 	QAbstractBarSeries *bar_series = NULL;
 	double maxvalue = 0.0, minvalue = 0.0;
-	
+
 	if(chart_type == 1) {
 		pie_series = new QPieSeries();
 	} else if(chart_type == 3) {
@@ -1077,17 +1103,20 @@ void CategoriesComparisonChart::updateDisplay() {
 			else legend_string = account->nameWithParent();
 			current_value = values[account];
 		}
-		legend_value = (current_value * 100) / value;
-		int deci = 0;
-		if(legend_value < 10.0 && legend_value > -10.0) {
-			legend_value = round(legend_value * 10.0) / 10.0;
-			deci = 1;
-		} else {
-			legend_value = round(legend_value);
-		}
 
 		if(chart_type == 1) {
-			QPieSlice *slice = pie_series->append(QString("%1 (%2%)").arg(legend_string).arg(currency->formatValue(legend_value, deci, false)), current_value);
+			int deci = 0;
+			if(valueButton->isChecked()) {
+				legend_value = current_value;
+				if(legend_value < 100.0) deci = currency->fractionalDigits();
+			} else {
+				legend_value = (current_value * 100) / value;
+				if(legend_value < 10.0 && legend_value > -10.0) {
+					legend_value = round(legend_value * 10.0) / 10.0;
+					deci = 1;
+				}
+			}
+			QPieSlice *slice = pie_series->append(QString("%1 (%2)").arg(legend_string).arg(valueButton->isChecked() ? currency->formatValue(legend_value, deci) : budget->formatValue(legend_value, deci) + "%"), current_value);
 			if(theme < 0) {
 				slice->setBrush(getPieBrush(index, account_order.size() > 0 ? account_order.size() : desc_order.size()));
 				slice->setLabelColor(Qt::black);
@@ -1096,7 +1125,7 @@ void CategoriesComparisonChart::updateDisplay() {
 			}
 			slice->setLabelArmLengthFactor(0.1);
 			slice->setExplodeDistanceFactor(0.1);
-			if(legend_value >= 8.0) {
+			if((current_value * 100) / value >= 8.0) {
 				slice->setLabelVisible(true);
 			} else {
 				show_legend = true;
@@ -1108,17 +1137,19 @@ void CategoriesComparisonChart::updateDisplay() {
 			if(current_value < minvalue) minvalue = current_value;
 			show_legend = true;
 		}
-		
+
 		if(chart_type == 2) index--;
 		else index++;
 	}
-	
+
 	chart->removeAllSeries();
-	
+
 	foreach(QAbstractAxis* axis, chart->axes()) {
 		chart->removeAxis(axis);
 	}
-	
+	axis_x = NULL;
+	axis_y = NULL;
+
 	if(theme < 0) {
 		chart->setBackgroundBrush(Qt::white);
 		chart->setTitleBrush(Qt::black);
@@ -1127,7 +1158,9 @@ void CategoriesComparisonChart::updateDisplay() {
 		chart->legend()->setColor(Qt::white);
 		chart->legend()->setLabelColor(Qt::black);
 	}
-	
+
+	chart->setLocalizeNumbers((chart_type == 1 && (budget->decimal_separator != "." || budget->decimal_separator == QLocale().decimalPoint())) || (chart_type != 1 && (budget->monetary_decimal_separator != "." || budget->monetary_decimal_separator == QLocale().decimalPoint() || ((maxvalue - minvalue) >= 50.0 && (budget->monetary_group_separator == QLocale().groupSeparator() || QLocale().groupSeparator() == ' ' || QLocale().groupSeparator() == 0x202F || QLocale().groupSeparator() == 0x2009)))));
+
 	if(chart_type == 1) {
 		series = pie_series;
 		pie_series->setPieSize(0.78);
@@ -1148,13 +1181,13 @@ void CategoriesComparisonChart::updateDisplay() {
 		v_axis->setRange(minvalue, maxvalue);
 		v_axis->setTickCount(y_lines + 1);
 		v_axis->setMinorTickCount(chart_type == 2 ? 0 : y_minor);
-		if(type == 3 || (maxvalue - minvalue) >= 50.0) v_axis->setLabelFormat(QString("%.0f"));
+		if((maxvalue - minvalue) >= 50.0) v_axis->setLabelFormat(QString("%.0f"));
 		else v_axis->setLabelFormat(QString("%.%1f").arg(QString::number(currency->fractionalDigits())));
-		
+
 		if(type == ACCOUNT_TYPE_ASSETS) v_axis->setTitleText(tr("Value") + QString(" (%1)").arg(currency->symbol(true)));
 		else if(type == ACCOUNT_TYPE_INCOMES) v_axis->setTitleText(tr("Income") + QString(" (%1)").arg(currency->symbol(true)));
 		else v_axis->setTitleText(tr("Cost") + QString(" (%1)").arg(currency->symbol(true)));
-		
+
 		if(theme < 0) {
 			v_axis->setLinePen(QPen(Qt::darkGray, 1));
 			v_axis->setLabelsColor(Qt::black);
@@ -1169,18 +1202,20 @@ void CategoriesComparisonChart::updateDisplay() {
 		}
 
 		if(chart_type == 3) {
-			chart->setAxisX(b_axis, series);
-			chart->setAxisY(v_axis, series);
+			axis_x = b_axis;
+			axis_y = v_axis;
 		} else {
-			chart->setAxisY(b_axis, series);
-			chart->setAxisX(v_axis, series);
+			axis_x = v_axis;
+			axis_y = b_axis;
 		}
-		
+		chart->addAxis(axis_x, Qt::AlignBottom);
+		chart->addAxis(axis_y, Qt::AlignLeft);
+
 		connect(bar_series, SIGNAL(hovered(bool, int, QBarSet*)), this, SLOT(onSeriesHovered(bool, int, QBarSet*)));
-		
+
 		show_legend = false;
 	}
-	
+
 	chart->setTitle(QString("<div align=\"center\"><font size=\"+2\"><b>%1</b></font></div>").arg(title_string));
 	if(show_legend) {
 #if (QT_CHARTS_VERSION >= QT_CHARTS_VERSION_CHECK(5, 7, 0))
@@ -1202,15 +1237,16 @@ void CategoriesComparisonChart::updateDisplay() {
 	QFontMetrics fm(legend_font);
 	int fh = fm.height();
 	int margin = 15 + fh;
-	
+
 	QVector<QGraphicsItem*> legend_texts;
-	
+
 	account = NULL;
 	int index = 0;
 	int text_width = 0;
 	while((index < account_order.count()) || (current_account && index < desc_order.size())) {
 		QString legend_string;
 		double legend_value = 0.0;
+		double current_value = 0.0;
 		if(account_order.isEmpty()) {
 			if(desc_order[index].isEmpty()) {
 				legend_string = tr("No description", "Referring to the transaction description property (transaction title/generic article name)");
@@ -1227,29 +1263,32 @@ void CategoriesComparisonChart::updateDisplay() {
 			else legend_string = account->nameWithParent();
 			legend_value = values[account];
 		}
-		legend_value = (legend_value * 100) / value;
 		int deci = 0;
-		if(legend_value < 10.0 && legend_value > -10.0) {
-			legend_value = round(legend_value * 10.0) / 10.0;
-			deci = 1;
+		if(valueButton->isChecked()) {
+			legend_value = current_value;
+			if(legend_value < 100.0) deci = currency->fractionalDigits();
 		} else {
-			legend_value = round(legend_value);
+			legend_value = (current_value * 100) / value;
+			if(legend_value < 10.0 && legend_value > -10.0) {
+				legend_value = round(legend_value * 10.0) / 10.0;
+				deci = 1;
+			}
 		}
-		QGraphicsSimpleTextItem *legend_text = new QGraphicsSimpleTextItem(QString("%1 (%2%)").arg(legend_string).arg(currency->formatValue(legend_value, deci, false)));
+		QGraphicsSimpleTextItem *legend_text = new QGraphicsSimpleTextItem(QString("%1 (%2%)").arg(legend_string).arg(valueButton->isChecked() ? currency->formatValue(legend_value, deci) : budget->formatValue(legend_value, deci) + "%"));
 		legend_text->setFont(legend_font);
 		legend_text->setBrush(Qt::black);
 		if(legend_text->boundingRect().width() > text_width) text_width = legend_text->boundingRect().width();
 		legend_texts << legend_text;
 		index++;
 	}
-	
+
 	QGraphicsTextItem *title_text = new QGraphicsTextItem();
 	title_text->setHtml(QString("<div align=\"center\"><font size=\"+2\"><b>%1</b></font></div>").arg(title_string));
 	title_text->setDefaultTextColor(Qt::black);
 	title_text->setFont(legend_font);
 	title_text->setPos(view->height() / 2 - title_text->boundingRect().width() / 2, margin);
 	scene->addItem(title_text);
-	
+
 	int diameter = view->height() - margin * 3 - title_text->boundingRect().height();
 	if(diameter + margin * 2 + fh * 1.3 + text_width + margin > view->width()) diameter = view->width() - (margin * 2 + fh * 1.3 + text_width + margin);
 	int legend_x = diameter + margin * 2;
@@ -1372,27 +1411,19 @@ void CategoriesComparisonChart::updateAccounts() {
 	if(curindex < sourceCombo->count()) sourceCombo->setCurrentIndex(curindex);
 	sourceCombo->blockSignals(false);
 	accountCombo->blockSignals(true);
-	AssetsAccount *current_assets = selectedAccount();
-	accountCombo->clear();
-	accountCombo->addItem(tr("All Accounts"), qVariantFromValue(NULL));
-	for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
-		AssetsAccount *account = *it;
-		if(account != budget->balancingAccount && account->accountType() != ASSETS_TYPE_SECURITIES) {
-			accountCombo->addItem(account->name(), qVariantFromValue((void*) account));
-		}
-	}
-	int index = 0;
-	if(current_assets) index = accountCombo->findData(qVariantFromValue((void*) current_assets));
-	if(index >= 0) accountCombo->setCurrentIndex(index);
-	else accountCombo->setCurrentIndex(0);
+	accountCombo->updateAccounts(ACCOUNT_TYPE_ASSETS);
 	accountCombo->blockSignals(false);
 	updateDisplay();
 }
-AssetsAccount *CategoriesComparisonChart::selectedAccount() {
-	if(!accountCombo->currentData().isValid()) return NULL;
-	return (AssetsAccount*) accountCombo->currentData().value<void*>();
-}
 
+void CategoriesComparisonChart::valueTypeToggled(int i, bool b) {
+	if(!b) return;
+	QSettings settings;
+	settings.beginGroup("CategoriesComparisonChart");
+	settings.setValue("valueType", i);
+	settings.endGroup();
+	updateDisplay();
+}
 #ifdef QT_CHARTS_LIB
 void CategoriesComparisonChart::themeChanged(int index) {
 	int theme = themeCombo->itemData(index).toInt();
@@ -1403,7 +1434,9 @@ void CategoriesComparisonChart::themeChanged(int index) {
 	settings.endGroup();
 	updateDisplay();
 }
-void CategoriesComparisonChart::typeChanged(int) {
+void CategoriesComparisonChart::typeChanged(int type) {
+	percentButton->setEnabled(type == 0);
+	valueButton->setEnabled(type == 0);
 	updateDisplay();
 }
 void CategoriesComparisonChart::sliceHovered(QPieSlice *slice, bool state) {
@@ -1435,7 +1468,9 @@ class PointLabel2 : public QGraphicsItem {
 
 	public:
 
-		PointLabel2(QChart *c) : QGraphicsItem(c), chart(c) {}
+		PointLabel2(QChart *c, QAbstractAxis *axis_x) : QGraphicsItem(c), chart(c), axis(axis_x) {}
+
+		void setAxis(QAbstractAxis *axis_x) {axis = axis_x;}
 
 		void setText(const QString &text) {
 			m_text = text;
@@ -1499,22 +1534,23 @@ class PointLabel2 : public QGraphicsItem {
 				path = path.simplified();
 			}
 			painter->setBrush(chart->backgroundBrush());
-			QPen pen = chart->axisX()->linePen();
+			QPen pen = axis->linePen();
 			pen.setWidth(1);
 			painter->setPen(pen);
 			painter->drawPath(path);
-			painter->setPen(QPen(chart->axisX()->labelsBrush().color()));
-			painter->setBrush(chart->axisX()->labelsBrush());
+			painter->setPen(QPen(axis->labelsBrush().color()));
+			painter->setBrush(axis->labelsBrush());
 			painter->drawText(m_textRect, m_text);
 		}
 
 	private:
-	
+
 		QString m_text;
 		QRectF m_textRect;
 		QRectF m_rect;
 		QPointF m_anchor;
 		QChart *chart;
+		QAbstractAxis *axis;
 
 };
 
@@ -1523,10 +1559,11 @@ void CategoriesComparisonChart::onSeriesHovered(bool state, int index, QBarSet *
 		QAbstractBarSeries *series = qobject_cast<QAbstractBarSeries*>(sender());
 		PointLabel2 *item;
 		if(!point_label) {
-			item = new PointLabel2(chart);
+			item = new PointLabel2(chart, axis_x);
 			point_label = item;
 		} else {
 			item = (PointLabel2*) point_label;
+			item->setAxis(axis_x);
 		}
 		QList<QBarSet*> barsets = series->barSets();
 		int set_index = barsets.indexOf(set);
@@ -1539,8 +1576,8 @@ void CategoriesComparisonChart::onSeriesHovered(bool state, int index, QBarSet *
 		pos_y += (set_index + 0.5) * (bar_width / barsets.count());
 		pos.setY(pos_y);
 		Currency *currency = budget->defaultCurrency();
-		if(selectedAccount()) currency = selectedAccount()->currency();
-		item->setText(tr("%1\nValue: %2").arg(((QBarCategoryAxis*) chart->axisY())->at(index)).arg(currency->formatValue(set->at(index))));
+		if(accountCombo->isEnabled() && accountCombo->selectedAccounts().count() == 1) currency = ((AssetsAccount*) accountCombo->selectedAccounts()[0])->currency();
+		item->setText(tr("%1\nValue: %2").arg(((QBarCategoryAxis*) axis_y)->at(index)).arg(currency->formatValue(set->at(index))));
 		item->setAnchor(pos);
 		item->setPos(pos + QPoint(10, -50));
 		item->setZValue(11);
