@@ -43,16 +43,16 @@
 #include "editcurrencydialog.h"
 #include "eqonomizevalueedit.h"
 
-EditAssetsAccountDialog::EditAssetsAccountDialog(Budget *budg, QWidget *parent, QString title, bool new_loan, int default_type, bool force_type) : QDialog(parent, 0), budget(budg) {
+EditAssetsAccountDialog::EditAssetsAccountDialog(Budget *budg, QWidget *parent, QString title, bool new_loan, int default_type, bool force_type, QString default_group) : QDialog(parent), budget(budg) {
 
 	setWindowTitle(title);
 	setModal(true);
-	
+
 	int row = 0;
 	prev_currency_index = 1;
 
 	QVBoxLayout *box1 = new QVBoxLayout(this);
-	
+
 	QGridLayout *grid = new QGridLayout();
 	box1->addLayout(grid);
 	if(new_loan) {
@@ -61,15 +61,42 @@ EditAssetsAccountDialog::EditAssetsAccountDialog(Budget *budg, QWidget *parent,
 		grid->addWidget(new QLabel(tr("Type:"), this), row, 0);
 		typeCombo = new QComboBox(this);
 		typeCombo->setEditable(false);
-		typeCombo->addItem(tr("Cash"));
-		typeCombo->addItem(tr("Transactional Account"));
-		typeCombo->addItem(tr("Savings Account"));
-		typeCombo->addItem(tr("Credit Card"));
-		typeCombo->addItem(tr("Debt"));
-		typeCombo->addItem(tr("Securities"));
-		typeCombo->addItem(tr("Other"));
+		typeCombo->addItem(budget->getAccountTypeName(ASSETS_TYPE_CASH, true));
+		typeCombo->addItem(budget->getAccountTypeName(ASSETS_TYPE_CURRENT, true));
+		typeCombo->addItem(budget->getAccountTypeName(ASSETS_TYPE_SAVINGS, true));
+		typeCombo->addItem(budget->getAccountTypeName(ASSETS_TYPE_CREDIT_CARD, true));
+		typeCombo->addItem(budget->getAccountTypeName(ASSETS_TYPE_LIABILITIES, true));
+		typeCombo->addItem(budget->getAccountTypeName(ASSETS_TYPE_SECURITIES, true));
+		typeCombo->addItem(budget->getAccountTypeName(ASSETS_TYPE_OTHER, true));
 		grid->addWidget(typeCombo, row, 1); row++;
 	}
+
+	grid->addWidget(new QLabel(tr("Group:"), this), row, 0);
+	groupCombo = new QComboBox(this);
+	groupCombo->setEditable(true);
+	QStringList groups;
+	if(!new_loan) {
+		groups << budget->getAccountTypeName(ASSETS_TYPE_CASH, true, true);
+		groups << budget->getAccountTypeName(ASSETS_TYPE_CURRENT, true, true);
+		groups << budget->getAccountTypeName(ASSETS_TYPE_SAVINGS, true, true);
+		groups << budget->getAccountTypeName(ASSETS_TYPE_SECURITIES, true, true);
+	}
+	groups << budget->getAccountTypeName(ASSETS_TYPE_CREDIT_CARD, true, true);
+	groups << budget->getAccountTypeName(ASSETS_TYPE_LIABILITIES, true, true);
+	for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
+		AssetsAccount *aaccount = *it;
+		if(!new_loan || aaccount->isLiabilities()) {
+			if(!aaccount->group().isEmpty()) groups << aaccount->group();
+		}
+	}
+	groups.removeDuplicates();
+	groups.sort(Qt::CaseInsensitive);
+	groupCombo->addItems(groups);
+	groupCombo->lineEdit()->setPlaceholderText(tr("no group"));
+	grid->addWidget(groupCombo, row, 1); row++;
+	if(!default_group.isEmpty()) groupCombo->setCurrentText(default_group);
+	else if(default_type >= 0 && !budget->getAccountTypeName(default_type, true, true).isEmpty()) groupCombo->setCurrentText(budget->getAccountTypeName(default_type, true, true));
+
 	if(default_type >= 0) {
 		switch(default_type) {
 			case ASSETS_TYPE_CASH: {typeCombo->setCurrentIndex(0); break;}
@@ -82,14 +109,14 @@ EditAssetsAccountDialog::EditAssetsAccountDialog(Budget *budg, QWidget *parent,
 		}
 		if(force_type) typeCombo->setEnabled(false);
 	}
-	
+
 	grid->addWidget(new QLabel(tr("Currency:"), this), row, 0);
 	currencyCombo = new QComboBox(this);
 	currencyCombo->setEditable(false);
 	grid->addWidget(currencyCombo, row, 1); row++;
 	editCurrencyButton = new QPushButton(tr("Edit"), this);
 	grid->addWidget(editCurrencyButton, row, 1, Qt::AlignRight); row++;
-	
+
 	grid->addWidget(new QLabel(tr("Name:"), this), row, 0);
 	nameEdit = new QLineEdit(this);
 	grid->addWidget(nameEdit, row, 1); row++;
@@ -112,7 +139,7 @@ EditAssetsAccountDialog::EditAssetsAccountDialog(Budget *budg, QWidget *parent,
 		accountCombo->updateAccounts();
 		grid->addWidget(accountCombo, row, 1); row++;
 		grid->addWidget(new QLabel(tr("Date:")), row, 0);
-		dateEdit = new QDateEdit(QDate::currentDate());
+		dateEdit = new EqonomizeDateEdit(QDate::currentDate());
 		dateEdit->setCalendarPopup(true);
 		grid->addWidget(dateEdit, row, 1);
 		row++;
@@ -146,16 +173,15 @@ EditAssetsAccountDialog::EditAssetsAccountDialog(Budget *budg, QWidget *parent,
 	}
 	nameEdit->setFocus();
 	current_account = NULL;
-	
+
 	updateCurrencyList(budget->defaultCurrency());
-	
+
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 	buttonBox->button(QDialogButtonBox::Ok)->setShortcut(Qt::CTRL | Qt::Key_Return);
-	buttonBox->button(QDialogButtonBox::Ok)->setDefault(true);
 	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
 	box1->addWidget(buttonBox);
-	
+
 	if(typeCombo) connect(typeCombo, SIGNAL(activated(int)), this, SLOT(typeActivated(int)));
 	if(currencyCombo) connect(currencyCombo, SIGNAL(activated(int)), this, SLOT(currencyActivated(int)));
 	if(closedButton) connect(closedButton, SIGNAL(toggled(bool)), this, SLOT(closedToggled(bool)));
@@ -174,7 +200,7 @@ void EditAssetsAccountDialog::updateCurrencyList(Currency *select_currency) {
 		} else {
 			currencyCombo->addItem(currency->code());
 		}
-		currencyCombo->setItemData(i, qVariantFromValue((void*) currency));
+		currencyCombo->setItemData(i, QVariant::fromValue((void*) currency));
 		if(currency == select_currency) {
 			prev_currency_index = i;
 			currencyCombo->setCurrentIndex(i);
@@ -189,7 +215,8 @@ void EditAssetsAccountDialog::transferToggled(bool b) {
 }
 void EditAssetsAccountDialog::closedToggled(bool b) {
 	if(b) budgetButton->setChecked(false);
-	budgetButton->setEnabled(!b);
+	int index = typeCombo->currentIndex();
+	budgetButton->setEnabled(!b && index != 5 && index != 4 && index != 3);
 }
 void EditAssetsAccountDialog::currencyActivated(int index) {
 	Currency *cur = NULL;
@@ -235,6 +262,17 @@ void EditAssetsAccountDialog::typeActivated(int index) {
 		}
 		return;
 	}
+	int i_type = -1;
+	switch(typeCombo->currentIndex()) {
+		case 0: {i_type = ASSETS_TYPE_CASH; break;}
+		case 1: {i_type = ASSETS_TYPE_CURRENT; break;}
+		case 2: {i_type = ASSETS_TYPE_SAVINGS; break;}
+		case 3: {i_type = ASSETS_TYPE_CREDIT_CARD; break;}
+		case 4: {i_type = ASSETS_TYPE_LIABILITIES; break;}
+		case 5: {i_type = ASSETS_TYPE_SECURITIES;  break;}
+		default: {i_type = ASSETS_TYPE_OTHER; break;}
+	}
+	if(budget->getAccountType(groupCombo->currentText(), true, true) != ASSETS_TYPE_OTHER) groupCombo->setCurrentText(budget->getAccountTypeName(i_type, true, true));
 	valueEdit->setEnabled(index != 5);
 	budgetButton->setEnabled(index != 5 && index != 4 && index != 3);
 	closedButton->setEnabled(index != 4);
@@ -267,6 +305,7 @@ AssetsAccount *EditAssetsAccountDialog::newAccount(Transaction **transfer) {
 	account->setCurrency((Currency*) currencyCombo->itemData(i).value<void*>());
 	if(maintainerEdit->isEnabled()) account->setMaintainer(maintainerEdit->text());
 	else account->setMaintainer("");
+	account->setGroup(groupCombo->currentText());
 	if(transfer && transferButton && transferButton->isChecked()) {
 		Transaction *trans = new Transfer(budget, valueEdit->value(), dateEdit->date(), account, (AssetsAccount*) accountCombo->currentAccount(), nameEdit->text());
 		*transfer = trans;
@@ -286,6 +325,7 @@ void EditAssetsAccountDialog::modifyAccount(AssetsAccount *account) {
 	account->setDescription(descriptionEdit->toPlainText());
 	account->setAsBudgetAccount(budgetButton->isChecked());
 	account->setClosed(closedButton->isChecked());
+	account->setGroup(groupCombo->currentText());
 	switch(typeCombo->currentIndex()) {
 		case 0: {account->setAccountType(ASSETS_TYPE_CASH); break;}
 		case 1: {account->setAccountType(ASSETS_TYPE_CURRENT); break;}
@@ -332,8 +372,8 @@ void EditAssetsAccountDialog::setAccount(AssetsAccount *account) {
 			break;
 		}
 		default: {typeCombo->setCurrentIndex(6); break;}
-	}	
-	for(int i = 0; i < currencyCombo->count(); i++) {		
+	}
+	for(int i = 0; i < currencyCombo->count(); i++) {
 		if(currencyCombo->itemData(i).value<void*>() == account->currency()) {
 			currencyCombo->setCurrentIndex(i);
 			prev_currency_index = i;
@@ -341,6 +381,7 @@ void EditAssetsAccountDialog::setAccount(AssetsAccount *account) {
 		}
 	}
 	typeActivated(typeCombo->currentIndex());
+	groupCombo->setCurrentText(account->group());
 }
 void EditAssetsAccountDialog::accept() {
 	QString sname = nameEdit->text().trimmed();
@@ -364,15 +405,15 @@ void EditAssetsAccountDialog::accept() {
 }
 
 
-EditIncomesAccountDialog::EditIncomesAccountDialog(Budget *budg, IncomesAccount *default_parent, QWidget *parent, QString title) : QDialog(parent, 0), budget(budg) {
+EditIncomesAccountDialog::EditIncomesAccountDialog(Budget *budg, IncomesAccount *default_parent, QWidget *parent, QString title) : QDialog(parent), budget(budg) {
 
 	setWindowTitle(title);
 	setModal(true);
-	
+
 	if(default_parent && default_parent->parentCategory()) default_parent = (IncomesAccount*) default_parent->parentCategory();
 
 	QVBoxLayout *box1 = new QVBoxLayout(this);
-	
+
 	QGridLayout *grid = new QGridLayout();
 	box1->addLayout(grid);
 	grid->addWidget(new QLabel(tr("Name:"), this), 0, 0);
@@ -389,12 +430,12 @@ EditIncomesAccountDialog::EditIncomesAccountDialog(Budget *budg, IncomesAccount
 		IncomesAccount *account = *it;
 		if(!account->parentCategory()) {
 			parentCombo->addItem(account->name());
-			parentCombo->setItemData(i, qVariantFromValue((void*) account));
+			parentCombo->setItemData(i, QVariant::fromValue((void*) account));
 			if(account == default_parent) parentCombo->setCurrentIndex(i);
 			i++;
 		}
 	}
-	
+
 	budgetButton = new QCheckBox(tr("Monthly budget:"), this);
 	budgetButton->setChecked(false);
 	grid->addWidget(budgetButton, 2, 0);
@@ -406,14 +447,13 @@ EditIncomesAccountDialog::EditIncomesAccountDialog(Budget *budg, IncomesAccount
 	grid->addWidget(descriptionEdit, 4, 0, 1, 2);
 	nameEdit->setFocus();
 	current_account = NULL;
-	
+
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 	buttonBox->button(QDialogButtonBox::Ok)->setShortcut(Qt::CTRL | Qt::Key_Return);
-	buttonBox->button(QDialogButtonBox::Ok)->setDefault(true);
 	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
 	box1->addWidget(buttonBox);
-	
+
 	connect(budgetButton, SIGNAL(toggled(bool)), this, SLOT(budgetEnabled(bool)));
 }
 IncomesAccount *EditIncomesAccountDialog::newAccount() {
@@ -471,15 +511,15 @@ void EditIncomesAccountDialog::accept() {
 	QDialog::accept();
 }
 
-EditExpensesAccountDialog::EditExpensesAccountDialog(Budget *budg, ExpensesAccount *default_parent, QWidget *parent, QString title) : QDialog(parent, 0), budget(budg) {
+EditExpensesAccountDialog::EditExpensesAccountDialog(Budget *budg, ExpensesAccount *default_parent, QWidget *parent, QString title) : QDialog(parent), budget(budg) {
 
 	setWindowTitle(title);
 	setModal(true);
-	
+
 	if(default_parent && default_parent->parentCategory()) default_parent = (ExpensesAccount*) default_parent->parentCategory();
 
 	QVBoxLayout *box1 = new QVBoxLayout(this);
-	
+
 	QGridLayout *grid = new QGridLayout();
 	box1->addLayout(grid);
 	grid->addWidget(new QLabel(tr("Name:"), this), 0, 0);
@@ -496,7 +536,7 @@ EditExpensesAccountDialog::EditExpensesAccountDialog(Budget *budg, ExpensesAccou
 		ExpensesAccount *account = *it;
 		if(!account->parentCategory()) {
 			parentCombo->addItem(account->name());
-			parentCombo->setItemData(i, qVariantFromValue((void*) account));
+			parentCombo->setItemData(i, QVariant::fromValue((void*) account));
 			if(account == default_parent) parentCombo->setCurrentIndex(i);
 			i++;
 		}
@@ -512,14 +552,13 @@ EditExpensesAccountDialog::EditExpensesAccountDialog(Budget *budg, ExpensesAccou
 	grid->addWidget(descriptionEdit, 4, 0, 1, 2);
 	nameEdit->setFocus();
 	current_account = NULL;
-	
+
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 	buttonBox->button(QDialogButtonBox::Ok)->setShortcut(Qt::CTRL | Qt::Key_Return);
-	buttonBox->button(QDialogButtonBox::Ok)->setDefault(true);
 	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
 	box1->addWidget(buttonBox);
-	
+
 	connect(budgetButton, SIGNAL(toggled(bool)), this, SLOT(budgetEnabled(bool)));
 }
 ExpensesAccount *EditExpensesAccountDialog::newAccount() {
