@@ -46,8 +46,9 @@
 #include <QPushButton>
 #include <QCheckBox>
 #include <QCompleter>
-#include <QDirModel>
+#include <QFileSystemModel>
 #include <QHash>
+#include <QMimeDatabase>
 
 #include "eqonomize.h"
 #include "qifimportexport.h"
@@ -57,6 +58,12 @@
 
 #include <QDebug>
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+#	define DATE_TO_MSECS(d) QDateTime(d.startOfDay()).toMSecsSinceEpoch()
+#else
+#	define DATE_TO_MSECS(d) QDateTime(d).toMSecsSinceEpoch()
+#endif
+
 extern QString last_document_directory;
 
 ImportQIFDialog::ImportQIFDialog(Budget *budg, QWidget *parent, bool extra_parameters) : QWizard(parent), budget(budg), b_extra(extra_parameters) {
@@ -75,7 +82,9 @@ ImportQIFDialog::ImportQIFDialog(Budget *budg, QWidget *parent, bool extra_param
 	QHBoxLayout *layout1h = new QHBoxLayout();
 	fileEdit = new QLineEdit(page1);
 	QCompleter *completer = new QCompleter(this);
-	completer->setModel(new QDirModel(completer));
+	QFileSystemModel *fsModel = new QFileSystemModel(completer);
+	fsModel->setRootPath(QString());
+	completer->setModel(fsModel);
 	fileEdit->setCompleter(completer);
 	layout1h->addWidget(fileEdit);
 	fileButton = new QPushButton(LOAD_ICON("document-open"), QString(), page1);
@@ -120,7 +129,7 @@ ImportQIFDialog::ImportQIFDialog(Budget *budg, QWidget *parent, bool extra_param
 	for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
 		AssetsAccount *account = *it;
 		if(account != budget->balancingAccount && account->accountType() != ASSETS_TYPE_SECURITIES) {
-			accountCombo->addItem(account->name(), qVariantFromValue((void*) account));
+			accountCombo->addItem(account->name(), QVariant::fromValue((void*) account));
 		}
 	}
 	layout3->addWidget(accountCombo, 0, 1);
@@ -128,7 +137,7 @@ ImportQIFDialog::ImportQIFDialog(Budget *budg, QWidget *parent, bool extra_param
 	openingBalanceEdit = new QLineEdit(page3);
 	openingBalanceEdit->setText("Opening Balance");
 	layout3->addWidget(openingBalanceEdit, 1, 1);
-	
+
 	QIFWizardPage *page4 = new QIFWizardPage();
 	page4->setTitle(tr("Import File"));
 	page4->setSubTitle(tr("No (further) issues were found. Press finish to import the selected QIF file."));
@@ -136,7 +145,7 @@ ImportQIFDialog::ImportQIFDialog(Budget *budg, QWidget *parent, bool extra_param
 	QGridLayout *layout4 = new QGridLayout(page4);
 	ignoreDuplicateTransactionsButton = new QCheckBox(tr("Ignore duplicate transactions"), page4);
 	layout4->addWidget(ignoreDuplicateTransactionsButton, 0, 0, 1, -1);
-	
+
 
 	setOption(QWizard::HaveHelpButton, false);
 
@@ -225,7 +234,8 @@ void ImportQIFDialog::onFileChanged(const QString &str) {
 	((QIFWizardPage*) page(0))->setComplete(!str.isEmpty());
 }
 void ImportQIFDialog::selectFile() {
-	QString url = QFileDialog::getOpenFileName(this, QString(), fileEdit->text().isEmpty() ? last_document_directory + "/" : fileEdit->text().trimmed(), "*.qif");
+	QMimeDatabase db;
+	QString url = QFileDialog::getOpenFileName(this, QString(), fileEdit->text().isEmpty() ? last_document_directory + "/" : fileEdit->text().trimmed(), db.mimeTypeForName("application/x-qw").filterString());
 	if(!url.isEmpty()) fileEdit->setText(url);
 }
 void ImportQIFDialog::nextClicked() {
@@ -406,10 +416,10 @@ void ImportQIFDialog::accept() {
 		file.close();
 		return;
 	}
-	
+
 	QFileInfo fileInfo(url);
 	last_document_directory = fileInfo.absoluteDir().absolutePath();
-	
+
 	QTextStream fstream(&file);
 	importQIF(fstream, false, qi, budget, ignoreDuplicateTransactionsButton->isChecked());
 	file.close();
@@ -444,7 +454,7 @@ void ImportQIFDialog::accept() {
 	return QWizard::reject();
 }
 
-ExportQIFDialog::ExportQIFDialog(Budget *budg, QWidget *parent, bool extra_parameters) : QDialog(parent, 0), budget(budg), b_extra(extra_parameters) {
+ExportQIFDialog::ExportQIFDialog(Budget *budg, QWidget *parent, bool extra_parameters) : QDialog(parent), budget(budg), b_extra(extra_parameters) {
 
 	setWindowTitle(tr("Export QIF File"));
 	setModal(true);
@@ -460,10 +470,10 @@ ExportQIFDialog::ExportQIFDialog(Budget *budg, QWidget *parent, bool extra_param
 	for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
 		AssetsAccount *account = *it;
 		if(account != budget->balancingAccount) {
-			accountCombo->addItem(account->name(), qVariantFromValue((void*) account));
+			accountCombo->addItem(account->name(), QVariant::fromValue((void*) account));
 		}
 	}
-	accountCombo->addItem(tr("All", "All accounts"), qVariantFromValue((void*) NULL));
+	accountCombo->addItem(tr("All", "All accounts"), QVariant::fromValue((void*) NULL));
 	grid->addWidget(accountCombo, 0, 1);
 	accountCombo->setFocus();
 
@@ -486,16 +496,17 @@ ExportQIFDialog::ExportQIFDialog(Budget *budg, QWidget *parent, bool extra_param
 	QHBoxLayout *layouth = new QHBoxLayout();
 	fileEdit = new QLineEdit(this);
 	QCompleter *completer = new QCompleter(this);
-	completer->setModel(new QDirModel(completer));
+	QFileSystemModel *fsModel = new QFileSystemModel(completer);
+	fsModel->setRootPath(QString());
+	completer->setModel(fsModel);
 	fileEdit->setCompleter(completer);
 	layouth->addWidget(fileEdit);
 	fileButton = new QPushButton(LOAD_ICON("document-open"), QString(), this);
 	layouth->addWidget(fileButton);
 	grid->addLayout(layouth, 3, 1);
-	
+
 	buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 	buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(Qt::CTRL | Qt::Key_Return);
-	buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
 	buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
 	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
@@ -512,8 +523,9 @@ void ExportQIFDialog::onFileChanged(const QString &str) {
 }
 void ExportQIFDialog::selectFile() {
 	QFileDialog fileDialog(this);
-	fileDialog.setNameFilter("*.qif");
-	fileDialog.setDefaultSuffix("qif");
+	QMimeDatabase db;
+	fileDialog.setNameFilter(db.mimeTypeForName("application/x-qw").filterString());
+	fileDialog.setDefaultSuffix(db.mimeTypeForName("application/x-qw").preferredSuffix());
 	fileDialog.setAcceptMode(QFileDialog::AcceptSave);
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
 	fileDialog.setSupportedSchemes(QStringList("file"));
@@ -555,10 +567,10 @@ void ExportQIFDialog::accept() {
 		QMessageBox::critical(this, tr("Error"), tr("Couldn't open file for writing."));
 		return;
 	}
-	
+
 	QFileInfo fileInfo(url);
 	last_document_directory = fileInfo.absoluteDir().absolutePath();
-	
+
 	QTextStream stream(&ofile);
 	exportQIF(stream, qi, budget, true);
 	if(!ofile.commit()) {
@@ -619,7 +631,11 @@ void testQIFDate(const QString &str, bool &p1, bool &p2, bool &p3, bool &p4, boo
 		lz = 1;
 		return;
 	}
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+	QStringList strlist = str.split(separator, Qt::SkipEmptyParts);
+#else
 	QStringList strlist = str.split(separator, QString::SkipEmptyParts);
+#endif
 	if(strlist.count() == 2 && (p1 || p2)) {
 		int i = strlist[1].indexOf('\'');
 		if(i >= 0) {
@@ -779,6 +795,7 @@ void importQIF(QTextStream &fstream, bool test, qif_info &qi, Budget *budget, bo
 			date_format += qi.lz == 0 ? "M" : "MM";
 		}
 	}
+	QMap<QDate, qint64> datestamps;
 	while(!line.isNull()) {
 		if(!line.isEmpty()) {
 			char field = line[0].toLatin1();
@@ -1158,7 +1175,7 @@ void importQIF(QTextStream &fstream, bool test, qif_info &qi, Budget *budget, bo
 												if(it != def_income_cats.constEnd()) parent_cat = *it;
 											}
 											if(parent_cat) b_exp = !b_exp;
-										} 
+										}
 										if(!parent_cat) {
 											if(!b_exp) parent_cat = new IncomesAccount(budget, current_split->category);
 											else parent_cat = new ExpensesAccount(budget, current_split->category);
@@ -1203,6 +1220,8 @@ void importQIF(QTextStream &fstream, bool test, qif_info &qi, Budget *budget, bo
 									}
 								}
 								if(!payee.isEmpty()) split->setPayee(payee);
+								split->setTimestamp(datestamps.contains(split->date()) ? datestamps[split->date()] + 1 : DATE_TO_MSECS(split->date()) / 1000);
+								datestamps[split->date()] = split->timestamp();
 								if(split->count() >= 2) {
 									budget->addSplitTransaction(split);
 									qi.transactions++;
@@ -1266,6 +1285,8 @@ void importQIF(QTextStream &fstream, bool test, qif_info &qi, Budget *budget, bo
 										qi.duplicates++;
 										delete tra;
 									} else {
+										tra->setTimestamp(datestamps.contains(tra->date()) ? datestamps[tra->date()] + 1 : DATE_TO_MSECS(tra->date()) / 1000);
+										datestamps[tra->date()] = tra->timestamp();
 										budget->addTransaction(tra);
 										transfers.append(tra);
 										qi.transactions++;
@@ -1290,7 +1311,7 @@ void importQIF(QTextStream &fstream, bool test, qif_info &qi, Budget *budget, bo
 									if(it != def_income_cats.constEnd()) parent_cat = *it;
 								}
 								if(parent_cat) b_exp = !b_exp;
-							} 
+							}
 							if(!parent_cat) {
 								if(!b_exp) parent_cat = new IncomesAccount(budget, category);
 								else parent_cat = new ExpensesAccount(budget, category);
@@ -1323,6 +1344,8 @@ void importQIF(QTextStream &fstream, bool test, qif_info &qi, Budget *budget, bo
 										qi.duplicates++;
 										delete exp;
 									} else {
+										exp->setTimestamp(datestamps.contains(exp->date()) ? datestamps[exp->date()] + 1 : DATE_TO_MSECS(exp->date()) / 1000);
+										datestamps[exp->date()] = exp->timestamp();
 										budget->addTransaction(exp);
 										qInfo() << exp->description();
 										qi.transactions++;
@@ -1339,7 +1362,9 @@ void importQIF(QTextStream &fstream, bool test, qif_info &qi, Budget *budget, bo
 									if(ignore_duplicates && budget->findDuplicateTransaction(inc)) {
 										qi.duplicates++;
 										delete inc;
-									} else {	
+									} else {
+										inc->setTimestamp(datestamps.contains(inc->date()) ? datestamps[inc->date()] + 1 : DATE_TO_MSECS(inc->date()) / 1000);
+										datestamps[inc->date()] = inc->timestamp();
 										budget->addTransaction(inc);
 										qInfo() << inc->description();
 										qi.transactions++;
@@ -1716,7 +1741,7 @@ void exportQIF(QTextStream &fstream, qif_info &qi, Budget *budget, bool export_c
 				Expense *exp = *it;
 				if(exp->from() == qi.current_account) {
 					ecats[exp->category()] = true;
-					
+
 				}
 			}
 			QMap<IncomesAccount*, bool>::iterator iit_e = icats.end();
@@ -1738,7 +1763,7 @@ void exportQIF(QTextStream &fstream, qif_info &qi, Budget *budget, bool export_c
 		bool first = true;
 		SplitTransaction *split = NULL;
 		for(TransactionList<Transaction*>::const_iterator it = budget->transactions.constBegin(); it != budget->transactions.constEnd(); ++it) {
-			Transaction *trans = *it;		
+			Transaction *trans = *it;
 			if(trans->fromAccount() == qi.current_account || trans->toAccount() == qi.current_account) {
 				if(first) {
 					exportQIFOpeningBalance(fstream, qi, qi.current_account, trans->date());
