@@ -44,24 +44,25 @@
 #include <QMessageBox>
 
 #include "budget.h"
+#include "eqonomizevalueedit.h"
 #include "recurrence.h"
 #include "recurrenceeditwidget.h"
 
-EditExceptionsDialog::EditExceptionsDialog(QWidget *parent)  : QDialog(parent, 0) {
+EditExceptionsDialog::EditExceptionsDialog(QWidget *parent)  : QDialog(parent) {
 
 	setWindowTitle(tr("Edit Exceptions"));
 	setModal(true);
 
 	QVBoxLayout *box1 = new QVBoxLayout(this);
-	
+
 	QGridLayout *layout = new QGridLayout();
 	box1->addLayout(layout);
-	
+
 	layout->addWidget(new QLabel(tr("Occurrences:"), this), 0, 0);
 	occurrencesList = new QListWidget(this);
 	occurrencesList->setSelectionMode(QListWidget::ExtendedSelection);
 	layout->addWidget(occurrencesList, 1, 0);
-	
+
 	QVBoxLayout *buttonsLayout = new QVBoxLayout();
 	layout->addLayout(buttonsLayout, 0, 1, -1, 1);
 	buttonsLayout->addStretch(1);
@@ -77,14 +78,13 @@ EditExceptionsDialog::EditExceptionsDialog(QWidget *parent)  : QDialog(parent, 0
 	exceptionsList = new QListWidget(this);
 	exceptionsList->setSelectionMode(QListWidget::ExtendedSelection);
 	layout->addWidget(exceptionsList, 1, 2);
-	
+
 	infoLabel = new QLabel(QString("<i>") + tr("Only the first fifty occurrences are shown.") + QString("</i>"), this);
 	infoLabel->hide();
 	layout->addWidget(infoLabel, 2, 0, 1, -1);
 
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-	buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(Qt::CTRL | Qt::Key_Return);
-	buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
+	buttonBox->button(QDialogButtonBox::Ok)->setShortcut(Qt::CTRL | Qt::Key_Return);
 	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
 	box1->addWidget(buttonBox);
@@ -180,13 +180,13 @@ void EditExceptionsDialog::deleteException() {
 	}
 }
 
-EditRangeDialog::EditRangeDialog(const QDate &startdate, QWidget *parent) : QDialog(parent, 0), date(startdate) {
+EditRangeDialog::EditRangeDialog(const QDate &startdate, QWidget *parent) : QDialog(parent), date(startdate) {
 
 	setWindowTitle(tr("Edit Recurrence Range"));
 	setModal(true);
 
 	QVBoxLayout *box1 = new QVBoxLayout(this);
-	
+
 	QGridLayout *rangeLayout = new QGridLayout();
 	box1->addLayout(rangeLayout);
 
@@ -210,15 +210,14 @@ EditRangeDialog::EditRangeDialog(const QDate &startdate, QWidget *parent) : QDia
 	endDateButton = new QRadioButton(tr("End on"), this);
 	rangeButtonGroup->addButton(endDateButton);
 	rangeLayout->addWidget(endDateButton, 3, 0);
-	endDateEdit = new QDateEdit(startdate, this);
+	endDateEdit = new EqonomizeDateEdit(startdate, this);
 	endDateEdit->setCalendarPopup(true);
 	rangeLayout->addWidget(endDateEdit, 3, 1, 1, 2);
 
 	setRecurrence(NULL);
-	
+
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-	buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(Qt::CTRL | Qt::Key_Return);
-	buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
+	buttonBox->button(QDialogButtonBox::Ok)->setShortcut(Qt::CTRL | Qt::Key_Return);
 	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
 	box1->addWidget(buttonBox);
@@ -365,9 +364,9 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 	weeklyFrequencyLayout->addStretch(1);
 	QHBoxLayout *weeklyDaysLayout = new QHBoxLayout();
 	weeklyRuleLayout->addLayout(weeklyDaysLayout);
-	int weekStart = QLocale().firstDayOfWeek();	
+	int weekStart = QLocale().firstDayOfWeek();
 	for(int i = 0; i < 7; ++i ) {
-		weeklyButtons[i] = new QCheckBox(QDate::shortDayName(i + 1, QDate::StandaloneFormat), weeklyRuleWidget);
+		weeklyButtons[i] = new QCheckBox(QLocale().standaloneDayName(i + 1, QLocale::ShortFormat), weeklyRuleWidget);
 	}
 	for(int i = weekStart - 1; i < 7; ++i ) {
 		weeklyDaysLayout->addWidget(weeklyButtons[i]);
@@ -455,7 +454,6 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 	monthlyWeekCombo->addItem(tr("2nd Last"));
 	monthlyWeekCombo->addItem(tr("3rd Last"));
 	monthlyWeekCombo->addItem(tr("4th Last"));
-	monthlyWeekCombo->addItem(tr("5th Last"));
 	monthlyWeekLayout->addWidget(monthlyWeekCombo);
 	monthlyDayOfWeekCombo = new QComboBox(monthlyRuleWidget);
 	monthlyWeekLayout->addWidget(monthlyDayOfWeekCombo);
@@ -564,16 +562,16 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 		int months2 = 12;
 		if(months2 > months) {
 			for(int i2 = months + 1; i2 <= months2; i2++) {
-				yearlyMonthCombo_week->addItem(QDate::longMonthName(i2, QDate::StandaloneFormat));
-				yearlyMonthCombo->addItem(QDate::longMonthName(i2, QDate::StandaloneFormat));
+				yearlyMonthCombo_week->addItem(QLocale().monthName(i2, QLocale::LongFormat));
+				yearlyMonthCombo->addItem(QLocale().monthName(i2, QLocale::LongFormat));
 			}
 			months = months2;
 		}
 		date = date.addYears(1);
 	}
 	for(int i = 1; i <= 7; i++) {
-		yearlyDayOfWeekCombo->addItem(QDate::longDayName(i, QDate::StandaloneFormat));
-		monthlyDayOfWeekCombo->addItem(QDate::longDayName(i, QDate::StandaloneFormat));
+		yearlyDayOfWeekCombo->addItem(QLocale().standaloneDayName(i));
+		monthlyDayOfWeekCombo->addItem(QLocale().standaloneDayName(i));
 	}
 
 	connect(typeCombo, SIGNAL(activated(int)), ruleStack, SLOT(setCurrentIndex(int)));
@@ -747,7 +745,7 @@ bool RecurrenceEditWidget::validValues() {
 		}
 		case 2: {
 			int i_frequency = monthlyFrequencyEdit->value();
-			if(i_frequency % 12 == 0) {					
+			if(i_frequency % 12 == 0) {
 				int i_dayofmonth = monthlyDayCombo->currentIndex() + 1;
 				int i_month = date.month();
 				if(i_dayofmonth <= 31 && ((i_month == 2 && i_dayofmonth > 29) || (i_dayofmonth > 30 && (i_month == 4 || i_month == 6 || i_month == 9 || i_month == 11)))) {
@@ -758,7 +756,7 @@ bool RecurrenceEditWidget::validValues() {
 			}
 			break;
 		}
-		case 3: {			
+		case 3: {
 			if(yearlyOnDayOfMonthButton->isChecked()) {
 				int i_frequency = yearlyFrequencyEdit->value();
 				int i_dayofmonth = yearlyDayOfMonthEdit->value();
