@@ -303,7 +303,7 @@ double Security::shares(const QDate &date, bool estimate, bool no_scheduled_shar
 			}
 		}
 	}
-	if(!b && estimate && date > QDate::currentDate() && reinvestedDividends.count() > 0) {		
+	if(!b && estimate && date > QDate::currentDate() && reinvestedDividends.count() > 0) {
 		QDate date1 = QDate::currentDate();
 		int days = date1.daysTo(date);
 		int days2 = 0;
@@ -330,9 +330,10 @@ double Security::shares(const QDate &date, bool estimate, bool no_scheduled_shar
 double Security::value() {
 	return shares() * getQuotation(QDate::currentDate());
 }
-double Security::value(const QDate &date, bool estimate, bool no_scheduled_shares) {
-	if(estimate && date > QDate::currentDate()) return shares(date, estimate, no_scheduled_shares) * expectedQuotation(date);
-	return shares(date, estimate, no_scheduled_shares) * getQuotation(date);
+double Security::value(const QDate &date, int estimate, bool no_scheduled_shares) {
+	if(estimate > 0 && date > QDate::currentDate()) return shares(date, true, no_scheduled_shares) * expectedQuotation(date);
+	else if(estimate < 0 && quotations.count() >= 2 && quotations.firstKey() < date && quotations.lastKey() > date) return shares(date, false, no_scheduled_shares) * expectedQuotation(date);
+	return shares(date, false, no_scheduled_shares) * getQuotation(date);
 }
 double Security::cost(const QDate &date, bool no_scheduled_shares, Currency *cur) {
 	if(!cur) cur = currency();
@@ -656,12 +657,12 @@ double Security::expectedQuotation(const QDate &date) {
 	if(it == it_begin) return 0.0;
 	--it;
 	if(it == it_begin) return it_begin.value();
-	if(date < it_begin.key()) {		
+	if(date < it_begin.key()) {
 		int days = date.daysTo(it_begin.key());
 		double q2 = expectedQuotation(it_begin.key().addDays(days));
 		return it_begin.value() * (it_begin.value() / q2);
 	}
-	if(it.key() < date) {		
+	if(it.key() < date) {
 		int days = it.key().daysTo(date);
 		double q1 = expectedQuotation(it.key().addDays(-days));
 		return it.value() * (it.value() / q1);
@@ -671,7 +672,7 @@ double Security::expectedQuotation(const QDate &date) {
 		if(it.key() < date) {
 			it_begin = it;
 			++it;
-			int days = it_begin.key().daysTo(date), days2 = it_begin.key().daysTo(it.key());
+			double days = it_begin.key().daysTo(date), days2 = it_begin.key().daysTo(it.key());
 			if(it.value() == it_begin.value()) return it.value();
 			double rate = it.value() / it_begin.value();
 			return it_begin.value() * pow(rate, days / days2);
