@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2006-2008, 2014, 2016 by Hanna Knutsson                 *
+ *   Copyright (C) 2006-2008, 2014, 2016-2020 by Hanna Knutsson            *
  *   hanna.knutsson@protonmail.com                                         *
  *                                                                         *
  *   This file is part of Eqonomize!.                                      *
@@ -49,18 +49,18 @@
 #include <cmath>
 
 TransactionFilterWidget::TransactionFilterWidget(bool extra_parameters, int transaction_type, Budget *budg, QWidget *parent) : QWidget(parent), transtype(transaction_type), budget(budg), b_extra(extra_parameters) {
-	payeeEdit = NULL;
+	tagCombo = NULL;
 	excludeSubsButton = NULL;
 	QGridLayout *filterLayout = new QGridLayout(this);
 	dateFromButton = new QCheckBox(tr("From:"), this);
 	dateFromButton->setChecked(false);
 	filterLayout->addWidget(dateFromButton, 0, 0);
-	dateFromEdit = new QDateEdit(QDate::currentDate(), this);
+	dateFromEdit = new EqonomizeDateEdit(QDate::currentDate(), this);
 	dateFromEdit->setCalendarPopup(true);
 	dateFromEdit->setEnabled(false);
 	filterLayout->addWidget(dateFromEdit, 0, 1);
 	filterLayout->addWidget(new QLabel(tr("To:"), this), 0, 2);
-	dateToEdit = new QDateEdit(QDate::currentDate(), this);
+	dateToEdit = new EqonomizeDateEdit(QDate::currentDate(), this);
 	dateToEdit->setCalendarPopup(true);
 	filterLayout->addWidget(dateToEdit, 0, 3);
 	QDate curdate = QDate::currentDate();
@@ -121,14 +121,10 @@ TransactionFilterWidget::TransactionFilterWidget(bool extra_parameters, int tran
 	descriptionEdit->completer()->setCaseSensitivity(Qt::CaseInsensitive);
 	filterLayout->addWidget(descriptionEdit, 3, 1);
 	if(b_extra && (transtype == TRANSACTION_TYPE_EXPENSE || transtype == TRANSACTION_TYPE_INCOME)) {
-		if(transtype == TRANSACTION_TYPE_INCOME) filterLayout->addWidget(new QLabel(tr("Payer:"), this), 3, 2);
-		else filterLayout->addWidget(new QLabel(tr("Payee:"), this), 3, 2);
-		payeeEdit = new QLineEdit(this);
-		payeeEdit->setCompleter(new QCompleter(this));
-		payeeEdit->completer()->setModel(new QStandardItemModel(this));
-		payeeEdit->completer()->setModelSorting(QCompleter::CaseInsensitivelySortedModel);
-		payeeEdit->completer()->setCaseSensitivity(Qt::CaseInsensitive);
-		filterLayout->addWidget(payeeEdit, 3, 3);
+		filterLayout->addWidget(new QLabel(tr("Tag:"), this), 3, 2);
+		tagCombo = new QComboBox(this);
+		tagCombo->setEditable(false);
+		filterLayout->addWidget(tagCombo, 3, 3);
 	}
 	QHBoxLayout *filterExcludeLayout = new QHBoxLayout();
 	group = new QButtonGroup(this);
@@ -149,20 +145,20 @@ TransactionFilterWidget::TransactionFilterWidget(bool extra_parameters, int tran
 	clearButton = new QPushButton(tr("Clear"), this);
 	clearButton->setEnabled(false);
 	filterExcludeLayout->addWidget(clearButton);
-	if(payeeEdit) {
+	if(b_extra) {
 		filterLayout->addLayout(filterExcludeLayout, 4, 0, 1, 4);
-		filterLayout->addItem(new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::Expanding), 5, 0, 1, 4);
 	} else {
 		filterLayout->addLayout(filterExcludeLayout, 3, 2, 1, 2);
 		filterLayout->addItem(new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::Expanding), 4, 0, 1, 4);
 	}
 
-	fromCombo->addItem(tr("All"), qVariantFromValue((void*) NULL));
-	toCombo->addItem(tr("All"), qVariantFromValue((void*) NULL));
+	fromCombo->addItem(tr("All"), QVariant::fromValue((void*) NULL));
+	toCombo->addItem(tr("All"), QVariant::fromValue((void*) NULL));
 
-	if(payeeEdit) {
-		connect(payeeEdit, SIGNAL(textChanged(const QString&)), this, SIGNAL(filter()));
-		connect(payeeEdit, SIGNAL(textChanged(const QString&)), this, SLOT(checkEnableClear()));
+	if(tagCombo) {
+		tagCombo->addItem(tr("All"));
+		connect(tagCombo, SIGNAL(activated(int)), this, SIGNAL(filter()));
+		connect(tagCombo, SIGNAL(activated(int)), this, SLOT(checkEnableClear()));
 	}
 	connect(clearButton, SIGNAL(clicked()), this, SLOT(clearFilter()));
 	connect(group, SIGNAL(buttonClicked(int)), this, SIGNAL(filter()));
@@ -214,7 +210,7 @@ void TransactionFilterWidget::clearFilter() {
 	fromCombo->blockSignals(true);
 	toCombo->blockSignals(true);
 	descriptionEdit->blockSignals(true);
-	if(payeeEdit) payeeEdit->blockSignals(true);
+	if(tagCombo) tagCombo->blockSignals(true);
 	dateFromButton->setChecked(false);
 	dateFromEdit->setEnabled(false);
 	to_date = QDate::currentDate();
@@ -225,8 +221,8 @@ void TransactionFilterWidget::clearFilter() {
 	maxEdit->setEnabled(false);
 	fromCombo->setCurrentIndex(0);
 	toCombo->setCurrentIndex(0);
-	descriptionEdit->setText(QString::null);
-	if(payeeEdit) payeeEdit->setText(QString::null);
+	descriptionEdit->setText(QString());
+	if(tagCombo) tagCombo->setCurrentIndex(0);
 	dateFromButton->blockSignals(false);
 	dateToEdit->blockSignals(false);
 	minButton->blockSignals(false);
@@ -234,18 +230,18 @@ void TransactionFilterWidget::clearFilter() {
 	fromCombo->blockSignals(false);
 	toCombo->blockSignals(false);
 	descriptionEdit->blockSignals(false);
-	if(payeeEdit) payeeEdit->blockSignals(false);
+	if(tagCombo) tagCombo->blockSignals(false);
 	clearButton->setEnabled(false);
 	emit filter();
 }
 void TransactionFilterWidget::checkEnableClear() {
-	clearButton->setEnabled(dateFromButton->isChecked() || minButton->isChecked() || maxButton->isChecked() || fromCombo->currentIndex() || toCombo->currentIndex() || !descriptionEdit->text().isEmpty() || (payeeEdit && !payeeEdit->text().isEmpty()) || to_date != QDate::currentDate());
+	clearButton->setEnabled(dateFromButton->isChecked() || minButton->isChecked() || maxButton->isChecked() || fromCombo->currentIndex() || toCombo->currentIndex() || !descriptionEdit->text().isEmpty() || (tagCombo && tagCombo->currentIndex()) || to_date != QDate::currentDate());
 }
-void TransactionFilterWidget::focusDescription() {
+void TransactionFilterWidget::focusFirst() {
 	descriptionEdit->setFocus();
 	descriptionEdit->selectAll();
 }
-void TransactionFilterWidget::setFilter(QDate fromdate, QDate todate, double min, double max, Account *from_account, Account *to_account, QString description, QString payee, bool exclude, bool exact_match, bool exclude_subs) {
+void TransactionFilterWidget::setFilter(QDate fromdate, QDate todate, double min, double max, Account *from_account, Account *to_account, QString description, QString tag, bool exclude, bool exact_match, bool exclude_subs) {
 	dateFromButton->blockSignals(true);
 	dateFromEdit->blockSignals(true);
 	dateToEdit->blockSignals(true);
@@ -256,7 +252,7 @@ void TransactionFilterWidget::setFilter(QDate fromdate, QDate todate, double min
 	fromCombo->blockSignals(true);
 	toCombo->blockSignals(true);
 	descriptionEdit->blockSignals(true);
-	if(payeeEdit) payeeEdit->blockSignals(true);
+	if(tagCombo) tagCombo->blockSignals(true);
 	excludeButton->blockSignals(true);
 	includeButton->blockSignals(true);
 	exactMatchButton->blockSignals(true);
@@ -293,7 +289,7 @@ void TransactionFilterWidget::setFilter(QDate fromdate, QDate todate, double min
 		maxEdit->setValue(max);
 	}
 	if(from_account) {
-		int i = fromCombo->findData(qVariantFromValue((void*) from_account));
+		int i = fromCombo->findData(QVariant::fromValue((void*) from_account));
 		if(i >= 0) {
 			fromCombo->setCurrentIndex(i);
 			emit fromActivated(from_account);
@@ -303,7 +299,7 @@ void TransactionFilterWidget::setFilter(QDate fromdate, QDate todate, double min
 		emit fromActivated(NULL);
 	}
 	if(to_account) {
-		int i = toCombo->findData(qVariantFromValue((void*) to_account));
+		int i = toCombo->findData(QVariant::fromValue((void*) to_account));
 		if(i >= 0) {
 			toCombo->setCurrentIndex(i);
 			emit toActivated(to_account);
@@ -313,7 +309,12 @@ void TransactionFilterWidget::setFilter(QDate fromdate, QDate todate, double min
 		emit toActivated(NULL);
 	}
 	descriptionEdit->setText(description);
-	if(payeeEdit) descriptionEdit->setText(payee);
+	if(tagCombo) {
+		int i = 0;
+		if(!tag.isEmpty()) i = tagCombo->findText(tag, Qt::MatchExactly);
+		if(i < 0) i = 0;
+		tagCombo->setCurrentIndex(i);
+	}
 	excludeButton->setChecked(exclude);
 	exactMatchButton->setChecked(exact_match);
 	if(excludeSubsButton) excludeSubsButton->setChecked(exclude_subs);
@@ -328,21 +329,29 @@ void TransactionFilterWidget::setFilter(QDate fromdate, QDate todate, double min
 	fromCombo->blockSignals(false);
 	toCombo->blockSignals(false);
 	descriptionEdit->blockSignals(false);
-	if(payeeEdit) payeeEdit->blockSignals(false);
+	if(tagCombo) tagCombo->blockSignals(false);
 	excludeButton->blockSignals(false);
 	includeButton->blockSignals(false);
 	exactMatchButton->blockSignals(false);
 	if(excludeSubsButton) excludeSubsButton->blockSignals(false);
 	emit filter();
 }
+void TransactionFilterWidget::updateTags() {
+	if(tagCombo) {
+		tagCombo->clear();
+		tagCombo->addItem(tr("All"));
+		tagCombo->addItems(budget->tags);
+		tagCombo->setCurrentIndex(0);
+	}
+}
 void TransactionFilterWidget::updateFromAccounts() {
 	fromCombo->clear();
-	fromCombo->addItem(tr("All"), qVariantFromValue((void*) NULL));
+	fromCombo->addItem(tr("All"), QVariant::fromValue((void*) NULL));
 	switch(transtype) {
 		case TRANSACTION_TYPE_TRANSFER: {
 			for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
 				Account *account = *it;
-				fromCombo->addItem(account->name(), qVariantFromValue((void*) account));
+				fromCombo->addItem(account->name(), QVariant::fromValue((void*) account));
 			}
 			break;
 		}
@@ -350,7 +359,7 @@ void TransactionFilterWidget::updateFromAccounts() {
 			for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
 				Account *account = *it;
 				if(account != budget->balancingAccount && ((AssetsAccount*) account)->accountType() != ASSETS_TYPE_SECURITIES) {
-					fromCombo->addItem(account->name(), qVariantFromValue((void*) account));
+					fromCombo->addItem(account->name(), QVariant::fromValue((void*) account));
 				}
 			}
 			break;
@@ -358,7 +367,7 @@ void TransactionFilterWidget::updateFromAccounts() {
 		case TRANSACTION_TYPE_INCOME: {
 			for(AccountList<IncomesAccount*>::const_iterator it = budget->incomesAccounts.constBegin(); it != budget->incomesAccounts.constEnd(); ++it) {
 				Account *account = *it;
-				fromCombo->addItem(account->name(), qVariantFromValue((void*) account));
+				fromCombo->addItem(account->name(), QVariant::fromValue((void*) account));
 			}
 			break;
 		}
@@ -367,12 +376,12 @@ void TransactionFilterWidget::updateFromAccounts() {
 }
 void TransactionFilterWidget::updateToAccounts() {
 	toCombo->clear();
-	toCombo->addItem(tr("All"), qVariantFromValue((void*) NULL));
+	toCombo->addItem(tr("All"), QVariant::fromValue((void*) NULL));
 	switch(transtype) {
 		case TRANSACTION_TYPE_TRANSFER: {
 			for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
 				Account *account = *it;
-				toCombo->addItem(account->name(), qVariantFromValue((void*) account));
+				toCombo->addItem(account->name(), QVariant::fromValue((void*) account));
 			}
 			break;
 		}
@@ -380,7 +389,7 @@ void TransactionFilterWidget::updateToAccounts() {
 			for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
 				Account *account = *it;
 				if(account != budget->balancingAccount) {
-					toCombo->addItem(account->name(), qVariantFromValue((void*) account));
+					toCombo->addItem(account->name(), QVariant::fromValue((void*) account));
 				}
 			}
 			break;
@@ -388,7 +397,7 @@ void TransactionFilterWidget::updateToAccounts() {
 		case TRANSACTION_TYPE_EXPENSE: {
 			for(AccountList<ExpensesAccount*>::const_iterator it = budget->expensesAccounts.constBegin(); it != budget->expensesAccounts.constEnd(); ++it) {
 				Account *account = *it;
-				toCombo->addItem(account->nameWithParent(), qVariantFromValue((void*) account));
+				toCombo->addItem(account->nameWithParent(), QVariant::fromValue((void*) account));
 			}
 			break;
 		}
@@ -398,20 +407,22 @@ void TransactionFilterWidget::updateToAccounts() {
 void TransactionFilterWidget::updateAccounts() {
 	updateFromAccounts();
 	updateToAccounts();
+	maxEdit->setCurrency(budget->defaultCurrency());
+	minEdit->setCurrency(budget->defaultCurrency());
 }
 bool TransactionFilterWidget::filterTransaction(Transactions *transs, bool checkdate) {
 	Transaction *trans = NULL;
 	MultiAccountTransaction *split = NULL;
 	switch(transs->generaltype()) {
 		case GENERAL_TRANSACTION_TYPE_SINGLE: {
-			trans = (Transaction*) transs; 
+			trans = (Transaction*) transs;
 			if(trans->parentSplit() && trans->parentSplit()->type() == SPLIT_TRANSACTION_TYPE_MULTIPLE_ACCOUNTS) return true;
 			if(trans->subtype() == TRANSACTION_SUBTYPE_REINVESTED_DIVIDEND && trans->value() == 0.0) return true;
 			break;
 		}
 		case GENERAL_TRANSACTION_TYPE_SPLIT: {
 			if(((SplitTransaction*) transs)->type() != SPLIT_TRANSACTION_TYPE_MULTIPLE_ACCOUNTS) return true;
-			split = (MultiAccountTransaction*) transs; 
+			split = (MultiAccountTransaction*) transs;
 			break;
 		}
 		case GENERAL_TRANSACTION_TYPE_SCHEDULE: {return filterTransaction(((ScheduledTransaction*) transs)->transaction(), checkdate);}
@@ -462,16 +473,26 @@ bool TransactionFilterWidget::filterTransaction(Transactions *transs, bool check
 				return true;
 			}
 		}
-		if(b_exact) {
-			if(!descriptionEdit->text().isEmpty() && transs->description().compare(descriptionEdit->text(), Qt::CaseInsensitive) != 0) {
-				return true;
-			}
-			if(payeeEdit && transtype == TRANSACTION_TYPE_EXPENSE && !payeeEdit->text().isEmpty() && ((Expense*) trans)->payee().compare(payeeEdit->text(), Qt::CaseInsensitive) != 0) {
-				if(split) {
-					bool b = false;
+		if(tagCombo && tagCombo->currentIndex() > 0 && !transs->hasTag(tagCombo->currentText(), true)) return true;
+		if(b_exact && !descriptionEdit->text().isEmpty()) {
+			bool b = transs->description().compare(descriptionEdit->text(), Qt::CaseInsensitive) != 0 && (tagCombo || !transs->hasTag(descriptionEdit->text(), true, true));
+			if(b_extra && b && transtype == TRANSACTION_TYPE_EXPENSE) {
+				b = ((Expense*) trans)->payee().compare(descriptionEdit->text(), Qt::CaseInsensitive) != 0;
+				if(b && split) {
 					for(int split_i = 1; split_i < split->count(); split_i++) {
-						if(((Expense*) split->at(split_i))->payee().compare(payeeEdit->text(), Qt::CaseInsensitive) == 0) {
-							b = true;
+						if(((Expense*) split->at(split_i))->payee().compare(descriptionEdit->text(), Qt::CaseInsensitive) == 0) {
+							b = false;
+							break;
+						}
+					}
+				}
+			}
+			if(b_extra && b && transtype == TRANSACTION_TYPE_INCOME) {
+				b = ((Income*) trans)->payer().compare(descriptionEdit->text(), Qt::CaseInsensitive) != 0;
+				if(b && split) {
+					for(int split_i = 1; split_i < split->count(); split_i++) {
+						if(((Income*) split->at(split_i))->payer().compare(descriptionEdit->text(), Qt::CaseInsensitive) == 0) {
+							b = false;
 							break;
 						}
 					}
@@ -480,52 +501,32 @@ bool TransactionFilterWidget::filterTransaction(Transactions *transs, bool check
 					return true;
 				}
 			}
-			if(payeeEdit && transtype == TRANSACTION_TYPE_INCOME && !payeeEdit->text().isEmpty() && ((Income*) trans)->payer().compare(payeeEdit->text(), Qt::CaseInsensitive) != 0) {
-				if(split) {
-					bool b = false;
+			if(b) return true;
+		} else if(!descriptionEdit->text().isEmpty()) {
+			bool b = !transs->description().contains(descriptionEdit->text(), Qt::CaseInsensitive) && !transs->comment().contains(descriptionEdit->text(), Qt::CaseInsensitive) && (tagCombo || !transs->hasTag(descriptionEdit->text(), true, true));
+			if(b_extra && b && transtype == TRANSACTION_TYPE_EXPENSE) {
+				b = !((Expense*) trans)->payee().contains(descriptionEdit->text(), Qt::CaseInsensitive);
+				if(b && split) {
 					for(int split_i = 1; split_i < split->count(); split_i++) {
-						if(((Income*) split->at(split_i))->payer().compare(payeeEdit->text(), Qt::CaseInsensitive) == 0) {
-							b = true;
+						if(((Expense*) split->at(split_i))->payee().contains(descriptionEdit->text(), Qt::CaseInsensitive)) {
+							b = false;
 							break;
 						}
 					}
-					if(!b) return true;
-				} else {
-					return true;
 				}
 			}
-		} else {
-			if(!descriptionEdit->text().isEmpty() && !transs->description().contains(descriptionEdit->text(), Qt::CaseInsensitive) && !transs->comment().contains(descriptionEdit->text(), Qt::CaseInsensitive)) {
-				return true;
-			}
-			if(payeeEdit && transtype == TRANSACTION_TYPE_EXPENSE && !payeeEdit->text().isEmpty() && !((Expense*) trans)->payee().contains(payeeEdit->text(), Qt::CaseInsensitive)) {
-				if(split) {
-					bool b = false;
+			if(b_extra && b && transtype == TRANSACTION_TYPE_INCOME) {
+				b = !((Income*) trans)->payer().contains(descriptionEdit->text(), Qt::CaseInsensitive);
+				if(b && split) {
 					for(int split_i = 1; split_i < split->count(); split_i++) {
-						if(((Expense*) split->at(split_i))->payee().contains(payeeEdit->text(), Qt::CaseInsensitive)) {
-							b = true;
+						if(((Income*) split->at(split_i))->payer().contains(descriptionEdit->text(), Qt::CaseInsensitive)) {
+							b = false;
 							break;
 						}
 					}
-					if(!b) return true;
-				} else {
-					return true;
-				}
-			}
-			if(payeeEdit && transtype == TRANSACTION_TYPE_INCOME && !payeeEdit->text().isEmpty() && !((Income*) trans)->payer().contains(payeeEdit->text(), Qt::CaseInsensitive)) {
-				if(split) {
-					bool b = false;
-					for(int split_i = 1; split_i < split->count(); split_i++) {
-						if(((Income*) split->at(split_i))->payer().contains(payeeEdit->text(), Qt::CaseInsensitive)) {
-							b = true;
-							break;
-						}
-					}
-					if(!b) return true;
-				} else {
-					return true;
 				}
 			}
+			if(b) return true;
 		}
 	} else {
 		Account *account = (Account*) toCombo->currentData().value<void*>();
@@ -536,15 +537,16 @@ bool TransactionFilterWidget::filterTransaction(Transactions *transs, bool check
 		if(fromCombo->currentIndex() > 0 && (account == trans->fromAccount() || (!b_exclude_subs && account == trans->fromAccount()->topAccount()))) {
 			if(!split || transtype != TRANSACTION_TYPE_EXPENSE || !split->account()) return true;
 		}
-		if(b_exact) {
-			if(!descriptionEdit->text().isEmpty() && transs->description().compare(descriptionEdit->text(), Qt::CaseInsensitive) == 0) {
+		if(tagCombo && tagCombo->currentIndex() > 0 && transs->hasTag(tagCombo->currentText(), true)) return true;
+		if(b_exact && !descriptionEdit->text().isEmpty()) {
+			if((transs->description().compare(descriptionEdit->text(), Qt::CaseInsensitive) == 0 || (!tagCombo && transs->hasTag(descriptionEdit->text(), true, true)))) {
 				return true;
 			}
-			if(payeeEdit && transtype == TRANSACTION_TYPE_EXPENSE  && !payeeEdit->text().isEmpty() && ((Expense*) trans)->payee().compare(payeeEdit->text(), Qt::CaseInsensitive) == 0) {
+			if(b_extra && transtype == TRANSACTION_TYPE_EXPENSE && ((Expense*) trans)->payee().compare(descriptionEdit->text(), Qt::CaseInsensitive) == 0) {
 				if(split) {
 					bool b = false;
 					for(int split_i = 1; split_i < split->count(); split_i++) {
-						if(((Expense*) split->at(split_i))->payee().compare(payeeEdit->text(), Qt::CaseInsensitive) != 0) {
+						if(((Expense*) split->at(split_i))->payee().compare(descriptionEdit->text(), Qt::CaseInsensitive) != 0) {
 							b = true;
 							break;
 						}
@@ -554,11 +556,11 @@ bool TransactionFilterWidget::filterTransaction(Transactions *transs, bool check
 					return true;
 				}
 			}
-			if(payeeEdit && transtype == TRANSACTION_TYPE_INCOME  && !payeeEdit->text().isEmpty() && ((Income*) trans)->payer().compare(payeeEdit->text(), Qt::CaseInsensitive) == 0) {
+			if(b_extra && transtype == TRANSACTION_TYPE_INCOME && ((Income*) trans)->payer().compare(descriptionEdit->text(), Qt::CaseInsensitive) == 0) {
 				if(split) {
 					bool b = false;
 					for(int split_i = 1; split_i < split->count(); split_i++) {
-						if(((Income*) split->at(split_i))->payer().compare(payeeEdit->text(), Qt::CaseInsensitive) != 0) {
+						if(((Income*) split->at(split_i))->payer().compare(descriptionEdit->text(), Qt::CaseInsensitive) != 0) {
 							b = true;
 							break;
 						}
@@ -568,15 +570,15 @@ bool TransactionFilterWidget::filterTransaction(Transactions *transs, bool check
 					return true;
 				}
 			}
-		} else {
-			if(!descriptionEdit->text().isEmpty() && transs->description().contains(descriptionEdit->text(), Qt::CaseInsensitive)) {
+		} else if(!descriptionEdit->text().isEmpty()) {
+			if(!descriptionEdit->text().isEmpty() && (transs->description().contains(descriptionEdit->text(), Qt::CaseInsensitive) || (!tagCombo && transs->hasTag(descriptionEdit->text(), true, true)))) {
 				return true;
 			}
-			if(payeeEdit && transtype == TRANSACTION_TYPE_EXPENSE  && !payeeEdit->text().isEmpty() && ((Expense*) trans)->payee().contains(payeeEdit->text(), Qt::CaseInsensitive)) {
+			if(b_extra && transtype == TRANSACTION_TYPE_EXPENSE && ((Expense*) trans)->payee().contains(descriptionEdit->text(), Qt::CaseInsensitive)) {
 				if(split) {
 					bool b = false;
 					for(int split_i = 1; split_i < split->count(); split_i++) {
-						if(!((Expense*) split->at(split_i))->payee().contains(payeeEdit->text(), Qt::CaseInsensitive)) {
+						if(!((Expense*) split->at(split_i))->payee().contains(descriptionEdit->text(), Qt::CaseInsensitive)) {
 							b = true;
 							break;
 						}
@@ -587,11 +589,11 @@ bool TransactionFilterWidget::filterTransaction(Transactions *transs, bool check
 				}
 				return true;
 			}
-			if(payeeEdit && transtype == TRANSACTION_TYPE_INCOME  && !payeeEdit->text().isEmpty() && ((Income*) trans)->payer().contains(payeeEdit->text(), Qt::CaseInsensitive)) {
+			if(b_extra && transtype == TRANSACTION_TYPE_INCOME  && ((Income*) trans)->payer().contains(descriptionEdit->text(), Qt::CaseInsensitive)) {
 				if(split) {
 					bool b = false;
 					for(int split_i = 1; split_i < split->count(); split_i++) {
-						if(!((Income*) split->at(split_i))->payer().contains(payeeEdit->text(), Qt::CaseInsensitive)) {
+						if(!((Income*) split->at(split_i))->payer().contains(descriptionEdit->text(), Qt::CaseInsensitive)) {
 							b = true;
 							break;
 						}
@@ -626,8 +628,6 @@ QDate TransactionFilterWidget::endDate() {
 }
 void TransactionFilterWidget::transactionsReset() {
 	((QStandardItemModel*) descriptionEdit->completer()->model())->clear();
-	if(payeeEdit) ((QStandardItemModel*) payeeEdit->completer()->model())->clear();
-	QStringList payee_list;
 	QStringList descr_list;
 	switch(transtype) {
 		case TRANSACTION_TYPE_EXPENSE: {
@@ -641,12 +641,12 @@ void TransactionFilterWidget::transactionsReset() {
 					((QStandardItemModel*) descriptionEdit->completer()->model())->appendRow(row);
 					descr_list << expense->description().toLower();
 				}
-				if(payeeEdit && !expense->payee().isEmpty() && !payee_list.contains(expense->payee(), Qt::CaseInsensitive)) {
+				if(b_extra && !expense->payee().isEmpty() && !descr_list.contains(expense->payee(), Qt::CaseInsensitive)) {
 					QList<QStandardItem*> row;
 					row << new QStandardItem(expense->payee());
 					row << new QStandardItem(expense->payee().toLower());
-					((QStandardItemModel*) payeeEdit->completer()->model())->appendRow(row);
-					payee_list << expense->payee().toLower();
+					((QStandardItemModel*) descriptionEdit->completer()->model())->appendRow(row);
+					descr_list << expense->payee().toLower();
 				}
 			}
 			break;
@@ -662,12 +662,12 @@ void TransactionFilterWidget::transactionsReset() {
 					((QStandardItemModel*) descriptionEdit->completer()->model())->appendRow(row);
 					descr_list << income->description().toLower();
 				}
-				if(payeeEdit && !income->security() && !income->payer().isEmpty() && !payee_list.contains(income->payer(), Qt::CaseInsensitive)) {
+				if(b_extra && !income->security() && !income->payer().isEmpty() && !descr_list.contains(income->payer(), Qt::CaseInsensitive)) {
 					QList<QStandardItem*> row;
 					row << new QStandardItem(income->payer());
 					row << new QStandardItem(income->payer().toLower());
-					((QStandardItemModel*) payeeEdit->completer()->model())->appendRow(row);
-					payee_list << income->payer().toLower();
+					((QStandardItemModel*) descriptionEdit->completer()->model())->appendRow(row);
+					descr_list << income->payer().toLower();
 				}
 			}
 			break;
@@ -688,8 +688,15 @@ void TransactionFilterWidget::transactionsReset() {
 		}
 		default: {}
 	}
+	if(transtype != TRANSACTION_TYPE_TRANSFER) {
+		for(int i = 0; i < budget->tags.count(); i++) {
+			QList<QStandardItem*> row;
+			row << new QStandardItem(budget->tags[i]);
+			row << new QStandardItem(budget->tags[i].toLower());
+			((QStandardItemModel*) descriptionEdit->completer()->model())->appendRow(row);
+		}
+	}
 	((QStandardItemModel*) descriptionEdit->completer()->model())->sort(1);
-	if(payeeEdit) ((QStandardItemModel*) payeeEdit->completer()->model())->sort(1);
 }
 void TransactionFilterWidget::transactionAdded(Transaction *trans) {
 	if(descriptionEdit && trans->type() == transtype && (transtype != TRANSACTION_TYPE_INCOME || !((Income*) trans)->security())) {
@@ -702,21 +709,21 @@ void TransactionFilterWidget::transactionAdded(Transaction *trans) {
 				((QStandardItemModel*) descriptionEdit->completer()->model())->sort(1);
 			}
 		}
-		if(payeeEdit && transtype == TRANSACTION_TYPE_EXPENSE && !((Expense*) trans)->payee().isEmpty()) {
-			if(((QStandardItemModel*) payeeEdit->completer()->model())->findItems(((Expense*) trans)->payee().toLower(), Qt::MatchExactly, 1).isEmpty()) {
+		if(b_extra && transtype == TRANSACTION_TYPE_EXPENSE && !((Expense*) trans)->payee().isEmpty()) {
+			if(((QStandardItemModel*) descriptionEdit->completer()->model())->findItems(((Expense*) trans)->payee().toLower(), Qt::MatchExactly, 1).isEmpty()) {
 				QList<QStandardItem*> row;
 				row << new QStandardItem(((Expense*) trans)->payee());
 				row << new QStandardItem(((Expense*) trans)->payee().toLower());
-				((QStandardItemModel*) payeeEdit->completer()->model())->appendRow(row);
-				((QStandardItemModel*) payeeEdit->completer()->model())->sort(1);
+				((QStandardItemModel*) descriptionEdit->completer()->model())->appendRow(row);
+				((QStandardItemModel*) descriptionEdit->completer()->model())->sort(1);
 			}
-		} else if(payeeEdit && transtype == TRANSACTION_TYPE_INCOME && !((Income*) trans)->security() && !((Income*) trans)->payer().isEmpty()) {
-			if(((QStandardItemModel*) payeeEdit->completer()->model())->findItems(((Income*) trans)->payer().toLower(), Qt::MatchExactly, 1).isEmpty()) {
+		} else if(b_extra && transtype == TRANSACTION_TYPE_INCOME && !((Income*) trans)->security() && !((Income*) trans)->payer().isEmpty()) {
+			if(((QStandardItemModel*) descriptionEdit->completer()->model())->findItems(((Income*) trans)->payer().toLower(), Qt::MatchExactly, 1).isEmpty()) {
 				QList<QStandardItem*> row;
 				row << new QStandardItem(((Income*) trans)->payer());
 				row << new QStandardItem(((Income*) trans)->payer().toLower());
-				((QStandardItemModel*) payeeEdit->completer()->model())->appendRow(row);
-				((QStandardItemModel*) payeeEdit->completer()->model())->sort(1);
+				((QStandardItemModel*) descriptionEdit->completer()->model())->appendRow(row);
+				((QStandardItemModel*) descriptionEdit->completer()->model())->sort(1);
 			}
 		}
 	}
@@ -731,8 +738,13 @@ void TransactionFilterWidget::toChanged(const QDate &date) {
 		error = true;
 	}
 	if(!error && dateFromEdit->date() > date) {
-		if(dateFromButton->isChecked()) {
+		/*if(dateFromButton->isChecked()) {
 			QMessageBox::critical(this, tr("Error"), tr("To date is before from date."));
+		}*/
+		if(budget->isFirstBudgetDay(to_date)) {
+			from_date = budget->firstBudgetDay(date);
+		} else {
+			from_date = date.addDays(-from_date.daysTo(to_date));
 		}
 		from_date = date;
 		dateFromEdit->blockSignals(true);
@@ -757,8 +769,13 @@ void TransactionFilterWidget::fromChanged(const QDate &date) {
 		error = true;
 	}
 	if(!error && date > dateToEdit->date()) {
-		QMessageBox::critical(this, tr("Error"), tr("From date is after to date."));
-		to_date = date;
+		//QMessageBox::critical(this, tr("Error"), tr("From date is after to date."));
+		if(budget->isLastBudgetDay(to_date)) {
+			to_date = budget->lastBudgetDay(date);
+		} else {
+			to_date = date.addDays(from_date.daysTo(to_date));
+		}
+		if(to_date > QDate::currentDate() && from_date <= QDate::currentDate()) to_date = QDate::currentDate();
 		dateToEdit->blockSignals(true);
 		dateToEdit->setDate(to_date);
 		dateToEdit->blockSignals(false);
